/**
 * @file globalInfos.cpp
 * @author Frederic MARTIN [martin-frederic@users.sourceforge.net]
 * @brief Acces a tout plein d'infos sur le jeu etat de la souris, du clavier, du focus, du temps ecoule, etc...
 */


#include "globalInfos.h"
#include <fstream>

GlobalInfos::GlobalInfos()
{
	plein_ecran=false;
	jeu_actif=true;
	possede_le_focus=true;
	clique_gauche_enfonce=false;
	largeur_ecran=LARGEUR_PAR_DEFAUT;
	hauteur_ecran=HAUTEUR_PAR_DEFAUT;
	dernier_temps=SDL_GetTicks();
	relacher_touches();
	IA_faible_priorite=true;
	fleche_de_selection=false;
	sol_disco=false;
	IA_ID=true;
}

GlobalInfos::~GlobalInfos()
{
	
}

void GlobalInfos::actualiser()
{

	temps_ecoule=SDL_GetTicks()-dernier_temps;
	dernier_temps=SDL_GetTicks();
	
	
	sur_clique_gauche_enfonce=false;
	sur_clique_gauche_relache=false;
	molette_haut=false;
	molette_bas=false;
	
			
	while( SDL_PollEvent(&event) )
	{
		switch( event.type )
		{
			// -------------------------------------- // 
			case SDL_ACTIVEEVENT:
				if ( event.active.gain == 0 )
					possede_le_focus = false;
				else
					possede_le_focus = true;
				break;
				
			// -------------------------------------- // 
			case SDL_KEYDOWN:
				if (event.key.keysym.sym<1024)
					etat_des_touches[event.key.keysym.sym]=true;
				break;
				
			// -------------------------------------- // 
			case SDL_KEYUP:
				if (event.key.keysym.sym<1024)
					etat_des_touches[event.key.keysym.sym]=false;
				break;
				
			// -------------------------------------- // 
			case SDL_MOUSEBUTTONDOWN:
				if (event.button.button==SDL_BUTTON_LEFT)
				{
					sur_clique_gauche_enfonce=true;
					clique_gauche_enfonce=true;
				}
				if (event.button.button==SDL_BUTTON_RIGHT)
				{
					sur_clique_droit_enfonce=true;
					clique_droit_enfonce=true;
				}
				if (event.button.button==SDL_BUTTON_WHEELUP)
					molette_haut=true;
				if (event.button.button==SDL_BUTTON_WHEELDOWN)
					molette_bas=true;
				break;
				
			// -------------------------------------- // 
			case SDL_MOUSEBUTTONUP:
				if (event.button.button==SDL_BUTTON_LEFT)
				{
					sur_clique_gauche_relache=true;
					clique_gauche_enfonce=false;
				}
				if (event.button.button==SDL_BUTTON_RIGHT)
				{
					sur_clique_droit_relache=true;
					clique_droit_enfonce=false;
				}
				break;
				
			// -------------------------------------- // 	
			case SDL_MOUSEMOTION:
				souris_x=event.motion.x;
				souris_y=event.motion.y;
				break;
				
			// -------------------------------------- // 
			case SDL_QUIT:
				jeu_actif=false;
				break;

			// -------------------------------------- // 
			default:
				break;
		}
	}
}


void GlobalInfos::changer_de_scene(int nouvelle_scene)
{
	if (nouvelle_scene>=0 && nouvelle_scene<NOMBRE_DE_SCENES)
		scene_actuelle=nouvelle_scene;
	else
	{
		scene_actuelle=0;// 0 = E_SCENE_INIT
		printf("Error : scene = %d does NOT exist.\nExiting...\n",nouvelle_scene);
		jeu_actif=false;
	}
}


char GlobalInfos::touche_est_enfonce(SDLKey touche)
{
	if (touche<1024)
		return etat_des_touches[touche];
	else
		return false;
}



void GlobalInfos::relacher_touches()
{
	//vide le tableau des touches enfoncées
	for(int i=0;i<1024;i++)
		etat_des_touches[i]=0;
}


bool GlobalInfos::loadGameConfiguration()
{
	std::ifstream file;
	std::string error_msg;
	std::string strFilename("config.ini");

	file.open(strFilename.c_str(), std::ios::in | std::ios::binary);
	if(!file)
	{
		error_msg="Error while loading configuration file : '" + strFilename + "'.\n";
		fprintf(stderr,error_msg.c_str());
		return false;
	}

	// parse all lines from the configuration file
	for(int line = 1; ; line++)
	{
		// read the next configuration line
		std::string strBuffer;
		std::getline(file, strBuffer);

		// stop if we reached the end of file
		if(file.eof()) break;

		// check if an error happend while reading from the file
		if(!file)
		{
			error_msg="Error while reading configuration file : '" + strFilename + "'.\n";
			fprintf(stderr,error_msg.c_str());
			return false;
		}

		// find the first non-whitespace character
		std::string::size_type pos;
		pos = strBuffer.find_first_not_of(" \t");

		// check for empty lines
		if((pos == std::string::npos) || (strBuffer[pos] == '\n') || (strBuffer[pos] == '\r') || (strBuffer[pos] == 0)) continue;

		// check for comment lines
		if(strBuffer[pos] == '#') continue;

		// get the key
		std::string strKey;
		strKey = strBuffer.substr(pos, strBuffer.find_first_of(" =\t\n\r", pos) - pos);
		pos += strKey.size();

		// get the '=' character
		pos = strBuffer.find_first_not_of(" \t", pos);
		if((pos == std::string::npos) || (strBuffer[pos] != '='))
		{
			error_msg="Syntax error in the configuration file : '" + strFilename + "'.\n";
			fprintf(stderr,error_msg.c_str());
			return false;
		}

		// find the first non-whitespace character after the '=' character
		pos = strBuffer.find_first_not_of(" \t", pos + 1);

		// get the data
		std::string strData;
		strData = strBuffer.substr(pos, strBuffer.find_first_of("\n\r", pos) - pos);

		// width
		if(strKey == "Width")
		{
			largeur_ecran = atoi(strData.c_str());
		}
		else if(strKey == "Height")
		{
			hauteur_ecran = atoi(strData.c_str());
		}
		else if(strKey == "Fullscreen")
		{
			if (strData=="yes")
				plein_ecran = true;
		}
		else if(strKey == "AI_priority_low")
		{
			if (strData=="no")
				IA_faible_priorite=false;
		}
		
		else if(strKey == "AI_level")
		{
			IA_difficulte = atoi(strData.c_str());
			if (IA_difficulte>3)
				IA_difficulte=3;
		}
		else if(strKey == "AI_time")
		{
			IA_temps = atoi(strData.c_str());
			if (IA_temps<1)
				IA_temps=1;
		}
		else if(strKey == "AI_IterativeDeepening")
		{
			if (strData=="no")
				IA_ID = false;
			else
				IA_ID = true;
		}
		else if(strKey == "Selection_arrow")
		{
			if (strData=="yes")
				fleche_de_selection=true;
			else
				fleche_de_selection=false;
		}
		else if (strKey == "Disco_floor")
		{
			if (strData=="yes")
				sol_disco=true;
			else
				sol_disco=false;
		}
		
	}

	// explicitely close the file
	file.close();
	return true;
}




