/**
 * @file globalInfos.h
 * @author Frederic MARTIN [martin-frederic@users.sourceforge.net]
 * @brief Acces a tout plein d'infos sur le jeu: etat de la souris, du clavier, du focus, du temps ecoule, etc...
 */


#ifndef __GLOBAL_INFOS_H__
#define __GLOBAL_INFOS_H__

#include <SDL/SDL.h>
#include <SDL/SDL_opengl.h>
#include <string>

// -------------------------------------------------------------------------- //

#define	VERSION_DU_JEU						"0.1"

#define	LARGEUR_PAR_DEFAUT					1024
#define HAUTEUR_PAR_DEFAUT					768

#define E_DONNEES_TEXTURE_TITRE				"data/textures/titre.png"
#define E_DONNEES_TEXTURE_FONT				"data/textures/font.png"
#define	E_DONNEES_TEXTURE_DAMIER			"data/textures/damier.jpg"
#define	E_DONNEES_TEXTURE_SOL				"data/textures/sol.jpg"
#define	E_DONNEES_TEXTURE_MURS				"data/textures/murs.jpg"
#define	E_DONNEES_TEXTURE_PORTE				"data/textures/porte.jpg"
#define E_DONNEES_TEXTURE_BOUTON_QUITTER	"data/textures/quitter.png"
#define E_DONNEES_TEXTURE_BOUTON_CAMERA		"data/textures/camera.png"
#define E_DONNEES_TEXTURE_BOUTON_FORCE		"data/textures/force.png"
#define E_DONNEES_TEXTURE_ICONE_BLENDER		"data/textures/blender.png"
#define E_DONNEES_TEXTURE_ICONE_TUX			"data/textures/tux.png"
#define E_DONNEES_TEXTURE_ICONE_GIMP		"data/textures/gimp.png"
#define E_DONNEES_TEXTURE_ICONE_KATE		"data/textures/kate.png"
#define E_DONNEES_TEXTURE_BOUTON_QUITTER2	"data/textures/exit.png"
#define E_DONNEES_TEXTURE_BOUTON_1_JOUEUR	"data/textures/1.png"
#define E_DONNEES_TEXTURE_BOUTON_2_JOUEURS	"data/textures/2.png"
#define E_DONNEES_TEXTURE_BOUTON_CREDITS	"data/textures/credits.png"

#define E_DONNEES_OBJET3D_TOUR				"data/objets3d/tour.lwo"
#define E_DONNEES_OBJET3D_CAVALIER			"data/objets3d/cavalier.lwo"
#define E_DONNEES_OBJET3D_FOU				"data/objets3d/fou.lwo"
#define E_DONNEES_OBJET3D_DAME				"data/objets3d/dame.lwo"
#define E_DONNEES_OBJET3D_ROI				"data/objets3d/roi.lwo"
#define E_DONNEES_OBJET3D_PION				"data/objets3d/pion.lwo"
#define E_DONNEES_OBJET3D_PLATEAU			"data/objets3d/plateau.lwo"
#define E_DONNEES_OBJET3D_FLECHE			"data/objets3d/fleche.lwo"
#define E_DONNEES_OBJET3D_TABLE				"data/objets3d/table.lwo"


#define	NOMBRE_DE_SCENES					4




/**
 * @class GlobalInfos: Acces a tout plein d'infos sur le jeu: etat de la souris, du clavier, du focus, du temps ecoule, etc...
 * Cette classe permet d'obtenir et/ou de modifier tout un tas d'informations sur le jeu. En fait il ne devrait
 * y avoir qu'une seule instance de cette classe dans tout le jeu.
 */
class GlobalInfos
{
	public:
		GlobalInfos();
		~GlobalInfos();
		void actualiser();
		void changer_de_scene(int nouvelle_scene);
		bool surCliqueGauche()								{return sur_clique_gauche_enfonce;}
		bool surCliqueGaucheRelache()						{return sur_clique_gauche_relache;}
		bool cliqueGaucheEnfonce()							{return clique_gauche_enfonce;}
		bool surCliqueDroit()								{return sur_clique_droit_enfonce;}
		bool surCliqueDroitRelache()						{return sur_clique_droit_relache;}
		bool cliqueDroitEnfonce()							{return clique_droit_enfonce;}
		bool surMoletteHaut()								{return molette_haut;}
		bool surMoletteBas()								{return molette_bas;}
		bool le_jeu_est_actif() 							{return jeu_actif;}
		int SourisX()										{return souris_x;}
		int SourisY()										{return souris_y;}
		void quitter_le_jeu()								{jeu_actif=false;}
		int sceneActuelle()									{return scene_actuelle;}
		Uint32 tempsEcoule()								{return temps_ecoule;}
		void changer_largeurEcran(int l)					{largeur_ecran=l;}
		void changer_hauteurEcran(int h)					{hauteur_ecran=h;}
		int largeurEcran()									{return largeur_ecran;}
		int hauteurEcran()									{return hauteur_ecran;}
		void mettre_en_plein_ecran(bool pleinEcran)			{plein_ecran=pleinEcran;}
		bool est_en_plein_ecran()							{return plein_ecran;}
		bool le_jeu_a_le_focus()							{return possede_le_focus;}
		char touche_est_enfonce(SDLKey touche);
		void relacher_touches();
		bool loadGameConfiguration();
		// ---------------------------------
		// Spécifique au jeu
		// ---------------------------------
		enum mode_de_jeu
		{
			JOUEUR_CONTRE_JOUEUR,
			JOUEUR_CONTRE_IA
		};
		void set_mode_de_jeu(unsigned char mode)			{mode_de_jeu=mode;}
		unsigned char get_mode_de_jeu()						{return mode_de_jeu;}
		void set_IA_faible_priorite(bool priorite_faible)	{IA_faible_priorite=priorite_faible;}
		bool get_IA_faible_priorite()						{return IA_faible_priorite;}
		void set_fleche_de_selection(bool fleche)			{fleche_de_selection=fleche;}
		bool get_fleche_de_selection()						{return fleche_de_selection;}
		void set_IA_difficulte(unsigned char difficulte)	{IA_difficulte=difficulte;}
		unsigned char get_IA_difficulte()					{return IA_difficulte;}
		void set_sol_disco(bool disco)						{sol_disco=disco;}
		bool get_sol_disco()								{return sol_disco;}
		int get_limite_de_temps_pour_IA()					{return IA_temps;}
		bool get_IA_ID()									{return IA_ID;}
	private:
		bool plein_ecran;
		bool jeu_actif;
		bool possede_le_focus;
		bool sur_clique_gauche_enfonce;
		bool sur_clique_gauche_relache;
		bool clique_gauche_enfonce;
		bool sur_clique_droit_enfonce;
		bool sur_clique_droit_relache;
		bool clique_droit_enfonce;
		bool molette_haut;
		bool molette_bas;
		int souris_x;
		int souris_y;
		int scene_actuelle;
		int largeur_ecran, hauteur_ecran;
		Uint32 temps_ecoule, dernier_temps;
		SDL_Event event;
		char etat_des_touches[1024];
		// ---------------------------------
		// Spécifique au jeu
		// ---------------------------------
		unsigned char mode_de_jeu;
		bool IA_faible_priorite;
		bool fleche_de_selection;
		unsigned char IA_difficulte;
		bool sol_disco;
		unsigned int IA_temps;
		bool IA_ID;
	
};


// Unique instance de GlobalInfos dans le jeu.
// C'est un objet global comme ça on peut y avoir acces partout.
extern GlobalInfos* pGlobalInfos;


#endif
