/**
 * @file gui.cpp
 * @author Frederic MARTIN [martin-frederic@users.sourceforge.net]
 * @brief Un GUI basique que j'ai créé il y a longtemps... :-)
 */
 
#include "gui.h"

/*----------------------------------------------------------------*/
/*-------------------------- CGui class --------------------------*/
/*----------------------------------------------------------------*/
CGui::CGui()
{
	infos.aTextDrawer	= NULL;
	infos.down		= NULL;
}


CGui::~CGui()
{
	
}

void CGui::Init(float window_width, float window_height, GLuint aFontTextureId)
{
	width=window_width;
	height=window_height;
	infos.aTextDrawer = new CTextDrawer;
	infos.aTextDrawer->SetFontTextureId(aFontTextureId);
	infos.aTextDrawer->Initialise();
}


void CGui::Destroy()
{
	infos.aTextDrawer->Destroy();
	delete infos.aTextDrawer;
	RemoveAllWidgets();
}


void CGui::AddWidget(CGuiWidget* aWidget)
{
	widgets.push_back(aWidget);
}

void CGui::RemoveWidget(CGuiWidget* aWidget)
{
	widgets.remove(aWidget);
}

void CGui::RemoveAllWidgets()
{
	widgets.clear();
}


void CGui::Update(int mouse_x, int mouse_y, bool left_mouse_button_is_down)
{
	infos.x=mouse_x;
	infos.y=mouse_y;
	infos.on_left_button_up=false;
	infos.on_left_button_down=false;
	if ((left_mouse_button_is_down==false)&&(infos.left_button_down==true))
		infos.on_left_button_up=true;
	if ((left_mouse_button_is_down==true)&&(infos.left_button_down==false))
		infos.on_left_button_down=true;
	infos.left_button_down=left_mouse_button_is_down;
	
	list<CGuiWidget*>::iterator i;

	// update (and render) all widgets
	for(i=widgets.begin(); i != widgets.end(); ++i)
	{
		(*i)->Update(&infos);
	}
	
	if (infos.on_left_button_up)
		infos.down=NULL;
}

// Switches to 2d (ortho projection) mode.
// Use in conjunction with Exit2d()
void CGui::Go2d()
{
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	glOrtho( 0.0f, width, height, 0.0f, 0.0f, 1.0f );
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();
	glDisable(GL_LIGHTING);
}


// Switches out of 2d (ortho projection) mode.
// Use in conjunction with Go2d()
void CGui::Exit2d()
{
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
	glEnable(GL_LIGHTING);
}



/*----------------------------------------------------------------*/
/*----------------------- CGuiWidget class -----------------------*/
/*----------------------------------------------------------------*/
CGuiWidget::CGuiWidget()
{
	x1=0;
	y1=0;	
}

CGuiWidget::~CGuiWidget()
{
	
}

void CGuiWidget::SetPos(int x, int y)
{
	x1=x;
	y1=y;
}


/*----------------------------------------------------------------*/
/*----------------------- CGuiButton class -----------------------*/
/*----------------------------------------------------------------*/

CGuiButton::CGuiButton(const string& atext)
{
	x2=60;
	y2=40;
	call_back_on_click=NULL;
	SetText(atext);
}

CGuiButton::~CGuiButton()
{
	
}

void CGuiButton::Update(CGuiInfos* infos)
{
	bool mouse_over=((infos->x>=x1)&&(infos->x<x2)&&(infos->y>=y1)&&(infos->y<=y2));
	if (infos->on_left_button_down && mouse_over)
	{
		infos->down=this;
		x1++;
		y1++;
		x2++;
		y2++;
	}
	else
	if (infos->on_left_button_up)
	{
		if (infos->down==this)
		{
			x1--;
			y1--;
			x2--;
			y2--;
			if (mouse_over)
			{
				// CALLBACK : On click
				if (call_back_on_click!=NULL)
					(*call_back_on_click)();
			}
		}
	}
	
	glEnable(GL_BLEND);
	if (mouse_over  && (infos->down==this || infos->down==NULL))
	{
		glColor4f(0.2f, 0.2f, 1.0f,0.8f);
	    glBegin(GL_QUADS);
	    	glVertex2f(x1, y1);
	        glVertex2f(x1, y2);
	        glVertex2f(x2, y2);
	    	glVertex2f(x2, y1);
	    glEnd();
	    glColor4f(0.4f, 0.4f, 1.0f, 1.0f);
	    glBegin(GL_LINE_LOOP);
	    	glVertex2f(x1, y1);
	        glVertex2f(x1, y2);
	        glVertex2f(x2, y2);
	        glVertex2f(x2, y1);
			glVertex2f(x1, y1);
    	   glEnd();
	}
	else
	{
		glColor4f(0.2f, 0.2f, 1.0f,0.55f);
		glBegin(GL_QUADS);
			glVertex2f(x1, y1);
			glVertex2f(x1, y2);
			glVertex2f(x2, y2);
			glVertex2f(x2, y1);
		glEnd();
		glColor4f(0.4f, 0.4f, 1.0f, 0.8f);
		glBegin(GL_LINE_LOOP);
			glVertex2f(x1, y1);
			glVertex2f(x1, y2);
			glVertex2f(x2, y2);
			glVertex2f(x2, y1);
			glVertex2f(x1, y1);
		glEnd();
	}
	glDisable(GL_BLEND);
	
	infos->aTextDrawer->SetColor(1.0f,1.0f,1.0f,1.0f);
	infos->aTextDrawer->PrintText(x1+textPos,y1+(y2-y1)/2-16,true,text.c_str());

}

void CGuiButton::SetText(const string& atext)
{
	text=atext;
	SetTextPos();
}

void CGuiButton::SetPos(int x, int y)
{
	int width=x2-x1;
	int height=y2-y1;
	x1=x;
	y1=y;
	x2=x+width;
	y2=y1+height;
}

void CGuiButton::SetSize(int width, int height)
{
	if (width>0)
		x2=x1+width;
	if (height>0)
		y2=y1+height;

	SetTextPos();
}

void CGuiButton::OnClick(void (*afunction)())
{
	call_back_on_click=afunction;
}

void CGuiButton::SetTextPos()
{
	string::size_type len;
	
	// if the label size is too small 
	// (ie: the text cannot fit in it) 
	// just resize it.
	len=text.length();
	if ((unsigned int)(x2-x1)<(10+16*len)) // 10 = offset , 16 = size of a letter
		x2=x1+10+16*len;
	
	// now text in set in the center
	len=10+(text.length())*16;// 10 = offset , 8 = size of a letter
	unsigned int labelWidth=(unsigned int)(x2-x1);
	textPos=(labelWidth-len)/2;
}




/*----------------------------------------------------------------*/
/*-------------------- CGuiButtonImage class ---------------------*/
/*----------------------------------------------------------------*/


CGuiButtonImage::CGuiButtonImage(const string& atext)
{
	
}

CGuiButtonImage::~CGuiButtonImage()
{
	
}


void CGuiButtonImage::Update(CGuiInfos* infos)
{
	bool mouse_over=((infos->x>=x1)&&(infos->x<x2)&&(infos->y>=y1)&&(infos->y<=y2));
	if (infos->on_left_button_down && mouse_over)
	{
		infos->down=this;
		x1++;
		y1++;
		x2++;
		y2++;
	}
	else
		if (infos->on_left_button_up)
	{
		if (infos->down==this)
		{
			x1--;
			y1--;
			x2--;
			y2--;
			if (mouse_over)
			{
				// CALLBACK : On click
				if (call_back_on_click!=NULL)
					(*call_back_on_click)();
			}
		}
	}
	
	glEnable(GL_BLEND);
	glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, id);
	
	if (mouse_over  && (infos->down==this || infos->down==NULL))
		glColor3f(1.0f, 1.0f, 1.0f);
	else
		glColor3f(0.92f, 0.92f, 0.92f);
	
	glBegin(GL_QUADS);
	glTexCoord2f(0.0f, 0.0f);		glVertex2i(x1, y1);
	glTexCoord2f(0.0f, 1.0f);		glVertex2i(x1, y2);
	glTexCoord2f(1.0f, 1.0f);		glVertex2i(x2, y2);
	glTexCoord2f(1.0f, 0.0f);		glVertex2i(x2, y1);
	glEnd();
	
	glDisable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);

}



void CGuiButtonImage::SetImage(GLuint id_texture)
{
	id=id_texture;
}


/*----------------------------------------------------------------*/
/*----------------------- CGuiLabel class ------------------------*/
/*----------------------------------------------------------------*/

CGuiLabel::CGuiLabel(const string& atext)
{
	mR = mG = mB = mA = 1.0f;
	speciaux=false;
	x2=60;
	y2=40;
	SetText(atext);
}

CGuiLabel::~CGuiLabel()
{}

void CGuiLabel::Update(CGuiInfos* infos)
{
	infos->aTextDrawer->SetColor(mR,mG,mB,mA);
	infos->aTextDrawer->PrintText(x1+textPos,y1+(y2-y1)/2-16,!speciaux,text.c_str());
}

void CGuiLabel::SetText(const string& atext)
{
	text=atext;
	SetTextPos();
}

// Sets the colour of the text.
void CGuiLabel::SetColor(float aR, float aG, float aB, float aA)
{
	mR = aR;
	mG = aG;
	mB = aB;
	mA = aA;
}

void CGuiLabel::SetPos(int x, int y)
{
	int width=x2-x1;
	int height=y2-y1;
	x1=x;
	y1=y;
	x2=x+width;
	y2=y1+height;
}

void CGuiLabel::SetSize(int width, int height)
{
	if (width>0)
		x2=x1+width;
	if (height>0)
		y2=y1+height;

	SetTextPos();
}

void CGuiLabel::SetTextPos()
{
	string::size_type len;
	
	// if the label size is too small 
	// (ie: the text cannot fit in it) 
	// just resize it.
	len=text.length();
	if ((unsigned int)(x2-x1)<(10+16*len)) // 10 = offset , 16 = size of a letter
		x2=x1+10+16*len;
	
	// now text in set in the center
	len=10+(text.length())*16;// 10 = offset , 16 = size of a letter
	unsigned int labelWidth=(unsigned int)(x2-x1);
	textPos=(labelWidth-len)/2;
}

/*----------------------------------------------------------------*/
/*----------------------- CTextDrawer class ----------------------*/
/*----------------------------------------------------------------*/
CTextDrawer::CTextDrawer()
{
	mBaseListId = 0;
	mR = mG = mB = mA = 1.0f;
}


CTextDrawer::~CTextDrawer()
{
}


// Call when finished.
void CTextDrawer::Destroy()
{
	glDeleteLists(mBaseListId, 256);// Delete All Display Lists
}

// Sets the font texture GL texture id.
// Param: aFontTextureId - A GL texture enum for the font texture.
void CTextDrawer::SetFontTextureId(GLuint aFontTextureId)
{
	mFontTexId = aFontTextureId;
}


// Build Our Font Display List
void CTextDrawer::Initialise()		
{
	float	cx;							// Holds Our X Character Coord
	float	cy;							// Holds Our Y Character Coord
	int		loop;

	mBaseListId = glGenLists(256);					// Creating 256 Display Lists
	for (loop=0; loop<256; loop++)
	{
		cx=((float)(loop%16))/16.0f;				// X Position Of Current Character
		cy=((float)(loop/16))/16.0f;				// Y Position Of Current Character

		// Why 0.0625f value is used for texture coordinates ?
		// for 256 pixels width picture : 0.0625 = 16/256 ; 16 = letter width ; 256 = image width
		glNewList(mBaseListId+loop, GL_COMPILE);			// Start Building A List
			glBegin(GL_QUADS);					// Use A Quad For Each Character
				glTexCoord2f(cx,cy-0.0625f);			// Texture Coord (Bottom Left)
				glVertex2i(0,0);				// Vertex Coord (Bottom Left)
				glTexCoord2f(cx+0.0625f,cy-0.0625f);		// Texture Coord (Bottom Right)
				glVertex2i(32,0);				// Vertex Coord (Bottom Right)
				glTexCoord2f(cx+0.0625f,cy);			// Texture Coord (Top Right)
				glVertex2i(32,32);				// Vertex Coord (Top Right)
				glTexCoord2f(cx,cy);				// Texture Coord (Top Left)
				glVertex2i(0,32);				// Vertex Coord (Top Left)
			glEnd();						// Done Building Our Quad (Character)
			glTranslatef(16.0f,0.0f,0.0f);				// Move To The Right Of The Character
		glEndList();							// Done Building The Display List
	}
}


// Sets the colour of the text.
void CTextDrawer::SetColor(float aR, float aG, float aB, float aA)
{
	mR = aR;
	mG = aG;
	mB = aB;
	mA = aA;
}

// Actually there are 2 fonts in the texture font.
// So "first" allow us to choose between the first or second font.
void CTextDrawer::PrintText(float x, float y, bool first, const char* text)	
{
    glEnable(GL_TEXTURE_2D);
	glBindTexture(GL_TEXTURE_2D, mFontTexId);

	glEnable(GL_BLEND);									// For transparent background
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	glPushMatrix();
		glTranslated(x, y, 0);							// Position The Text (0,0 - Top Left)
		
		if (first)
			glListBase(mBaseListId + 16);
		else
			glListBase(mBaseListId + 128+16);

		glColor4f(mR, mG, mB, mA);						// Set text colour.

		glCallLists(strlen(text), GL_BYTE, text);		// Write The Text To The Screen
	glPopMatrix();


    glDisable(GL_TEXTURE_2D);
	glDisable(GL_BLEND);
}

