/**
 * @file gui.h
 * @author Frederic MARTIN [martin-frederic@users.sourceforge.net]
 * @brief Un GUI basique que j'ai créé il y a longtemps... :-)
 */

#ifndef __GUI_H__
#define __GUI_H__

#include <SDL/SDL_opengl.h>
#include <list>
#include <string>


using namespace std;


class CGui;
class CGuiWidget;
//class CGuiWindow;
//class CGuiEdit;
class CGuiButton;
class CGuiLabel;
//class CGuiCheckBox;
//class CGuiRadioButton;
class CTextDrawer;


struct CGuiInfos
{
	CTextDrawer* aTextDrawer;	//!< A text drawer instance which allows us to draw some text on the screen.
	int x,y;					//!< Mouse coordinates.
	bool left_button_down;		//!< If the left mouse button is down, left_button_down is true.
	bool on_left_button_up;		//!< If the left button is no more down (ie: just up) this variable is true.
	bool on_left_button_down;	//!< If the left button is no more up (ie: just down) this variable is true.
	CGuiWidget* down;			//!< It stores the widget where the mouse button has been down
};


class CGui
{
	public:
		CGui();
		~CGui();
		void Go2d();
		void Exit2d();
		void Init(float window_width, float window_height, GLuint aFontTextureId);
		void Destroy();
		void AddWidget(CGuiWidget* aWidget);
		void RemoveWidget(CGuiWidget* aWidget);
		void RemoveAllWidgets();
		void Update(int mouse_x=0, int mouse_y=0, bool left_mouse_button_is_down=false);
		
	private:
		list<CGuiWidget*> widgets;
		CGuiInfos infos;
		float width,height;
};


class CGuiWidget
{
	public:
		CGuiWidget();
		virtual ~CGuiWidget();
		/**
		*	Update the widget.
		*	In this function the widget is drawn and you can also check some input states (mouse or keyboard)
		*	to do something in particular (like call a "call-back" function or something)
		*	@param infos is a CGuiInfos structure. See the CGuiInfos structure for more informations.
		*/
		virtual void Update(CGuiInfos* infos)=0;
		/**
		*	All the widgets have a position.
		*	This method allows the user of the class to change the widget's position.
		*	@param x is the x position of the widget on the screen
		*	@param y is the y position of the widget on the screen
		*/
		virtual void SetPos(int x, int y);
		int getX() {return x1;}
		int getY() {return y1;}
		
	protected:
		int x1,y1;
	
};


class CGuiButton: public CGuiWidget
{
	public:
		CGuiButton(const string& atext="");
		virtual ~CGuiButton();
		virtual void Update(CGuiInfos* infos);
		void SetText(const string& atext);
		void SetSize(int width, int height);
		void SetPos(int x, int y);
		void OnClick(void (*afunction)());
		
	protected:
		string text;
		int x2,y2;
		int textPos;
		void SetTextPos();// Put the text at the middle of the button
		void (*call_back_on_click)(); // CALL BACK function
	
};


class CGuiButtonImage: public CGuiButton
{
	public:
		CGuiButtonImage(const string& atext="");
		virtual ~CGuiButtonImage();
		virtual void Update(CGuiInfos* infos);
		void SetImage(GLuint id_texture);
		
	private:
		GLuint id;
	
};

class CGuiLabel: public CGuiWidget
{
	public:
		CGuiLabel(const string& atext="");
		virtual ~CGuiLabel();
		virtual void Update(CGuiInfos* infos);
		void SetText(const string& atext);
		void SetPos(int x, int y);
		void SetColor(float aR, float aG, float aB, float aA=1.0f);
		
		void SetAlpha(float aA){mA=aA;}
		void SetSpeciaux(bool _speciaux){speciaux=_speciaux;};
		void SetSize(int width, int height);
		
	protected:
		string text;
		int x2,y2;
		int textPos;
		void SetTextPos();// Put the text at the middle of the label
		float mR, mG, mB, mA;	// Color.
		bool speciaux;
		
};


class CTextDrawer
{
	public:
		CTextDrawer();
		~CTextDrawer();
		void Initialise();
		void Destroy();
		void PrintText(float aX, float aY, bool first, const char* text);
		void SetFontTextureId(GLuint aFontTextureId);
		void SetColor(float aR, float aG, float aB, float aA);

	protected:
		GLuint mBaseListId;		// Start list id
		GLuint mFontTexId;		// GL font texture id.
		float mR, mG, mB, mA;	// Color.
};




#endif
