/**
 * @file models.cpp
 * @author Frederic MARTIN [martin-frederic@users.sourceforge.net]
 * @brief Les objets (modèles) 3D (pièces, damier, etc.)
 */
 

#include <SDL/SDL_opengl.h>
#include <math.h>
#include "models.h"


void dessiner_modele_lightWave(const CLightWaveObject& model);
void dessiner_damier();
void dessiner_case();
void dessiner_sol();
void dessiner_murs();
void dessiner_porte();

CLightWaveObject objTour;
CLightWaveObject objCavalier;
CLightWaveObject objFou;
CLightWaveObject objDame;
CLightWaveObject objRoi;
CLightWaveObject objPion;
CLightWaveObject objPlateau;
CLightWaveObject objFleche;
CLightWaveObject objTable;


bool charger_objets_3d()
{
	if(!objTour.LoadModel(E_DONNEES_OBJET3D_TOUR))
	{
		printf("Loading of "E_DONNEES_OBJET3D_TOUR" failed\n");
		return false;
	}
	
	if(!objCavalier.LoadModel(E_DONNEES_OBJET3D_CAVALIER)) 	
	{
		printf("Loading of "E_DONNEES_OBJET3D_CAVALIER" failed\n");
		return false;
	}
	
	if(!objFou.LoadModel(E_DONNEES_OBJET3D_FOU)) 	
	{
		printf("Loading of "E_DONNEES_OBJET3D_FOU" failed\n");
		return false;
	}
	
	if(!objDame.LoadModel(E_DONNEES_OBJET3D_DAME)) 	
	{
		printf("Loading of "E_DONNEES_OBJET3D_DAME" failed\n");
		return false;
	}
	
	if(!objRoi.LoadModel(E_DONNEES_OBJET3D_ROI)) 	
	{
		printf("Loading of "E_DONNEES_OBJET3D_ROI" failed\n");
		return false;
	}
	
	if(!objPion.LoadModel(E_DONNEES_OBJET3D_PION)) 	
	{
		printf("Loading of "E_DONNEES_OBJET3D_PION" failed\n");
		return false;
	}
	
	if(!objPlateau.LoadModel(E_DONNEES_OBJET3D_PLATEAU)) 	
	{
		printf("Loading of "E_DONNEES_OBJET3D_PLATEAU" failed\n");
		return false;
	}
	
	if(!objFleche.LoadModel(E_DONNEES_OBJET3D_FLECHE)) 	
	{
		printf("Loading of "E_DONNEES_OBJET3D_FLECHE" failed\n");
		return false;
	}
	
	if(!objTable.LoadModel(E_DONNEES_OBJET3D_TABLE)) 	
	{
		printf("Loading of "E_DONNEES_OBJET3D_TABLE" failed\n");
		return false;
	}
	return true;
}

void decharger_objets_3d()
{
	objTour.Shutdown();
	objCavalier.Shutdown();
	objFou.Shutdown();
	objDame.Shutdown();
	objRoi.Shutdown();
	objPion.Shutdown();
	objPlateau.Shutdown();
	objFleche.Shutdown();
	objTable.Shutdown();
}


GLuint genererla_liste_des_modeles(void)
{
	GLuint liste=glGenLists(TOTAL_NB_OBJ);
  
	glNewList(liste+OBJ_ROI, GL_COMPILE);
	dessiner_modele_lightWave(objRoi);
	glEndList();

	glNewList(liste+OBJ_DAME, GL_COMPILE);
	dessiner_modele_lightWave(objDame);
	glEndList();

	glNewList(liste+OBJ_FOU, GL_COMPILE);
	dessiner_modele_lightWave(objFou);
	glEndList();

	glNewList(liste+OBJ_CAVALIER, GL_COMPILE);
	dessiner_modele_lightWave(objCavalier);
	glEndList();

	glNewList(liste+OBJ_TOUR, GL_COMPILE);
	dessiner_modele_lightWave(objTour);
	glEndList();

	glNewList(liste+OBJ_PION, GL_COMPILE);
	dessiner_modele_lightWave(objPion);
	glEndList();
	
	glNewList(liste+OBJ_FLECHE, GL_COMPILE);
	dessiner_modele_lightWave(objFleche);
	glEndList();
  
	glNewList(liste+OBJ_DAMIER,GL_COMPILE);
	dessiner_damier();
	glEndList();
  
	glNewList(liste+OBJ_CASE,GL_COMPILE);
	dessiner_case();
	glEndList();
	
	glNewList(liste+OBJ_PLATEAU,GL_COMPILE);
	dessiner_modele_lightWave(objPlateau);
	glEndList();
	
	glNewList(liste+OBJ_TABLE,GL_COMPILE);
	dessiner_modele_lightWave(objTable);
	glEndList();
  
	glNewList(liste+OBJ_SOL,GL_COMPILE);
	dessiner_sol();
	glEndList();
	
	glNewList(liste+OBJ_MURS,GL_COMPILE);
	dessiner_murs();
	glEndList();	
	
	glNewList(liste+OBJ_PORTE,GL_COMPILE);
	dessiner_porte();
	glEndList();
	
	
	return liste;
}



void dessiner_modele_lightWave(const CLightWaveObject& model)
{
	// Pointers to the surface, polygon list, and vertex point.  These will be used
	// to draw the object.
	CVector4 *points;
	stSurface *surface;
	stPolygon *polygons = model.m_polygons;

	for(int i = 0; i < model.m_totalPolygons; i++)
	{
		if(polygons->totalPoints == 3) glBegin(GL_TRIANGLES);
		else if(polygons->totalPoints == 4) glBegin(GL_QUADS);
		else glBegin(GL_POLYGON);

		// Get this polygon's surface information.
		surface = &model.m_surfaces[polygons->surfaceIndex];

         // Set this poly's color and normal.
		glColor3f(surface->red, surface->green, surface->blue);
		//glNormal3f(-polygons->normal.x, -polygons->normal.y, -polygons->normal.z);

         // Here we draw a surface.  Get each point that makes up this shape and draw them.
		for(int j = 0; j < polygons->totalPoints; j++)
		{
			points = &model.m_normals[polygons->pointIndex[j]];
			glNormal3f(-points->x, -points->y, -points->z);
			points = &model.m_vertices[polygons->pointIndex[j]];
			glVertex3f(points->x, points->y, points->z);
		}

         // Move to the next polygon.
		polygons++;

		glEnd();
	}
}



void dessiner_damier()
{
	glBegin(GL_QUADS);

	glNormal3f(0.0f, 1.0f, 0.0f);
	
	glTexCoord2f(0.0f, 0.0f);
	glVertex3f(-1.0f, 0.0f, -1.0f);
	
	glTexCoord2f(1.0f, 0.0f);
	glVertex3f(15.0f, 0.0f, -1.0f);
	
	glTexCoord2f(1.0f, 1.0f);
	glVertex3f(15.0f, 0.0f, 15.0f);
	
	glTexCoord2f(0.0f, 1.0f);
	glVertex3f(-1.0f, 0.0f, 15.0f);
	
	glEnd();
}


void dessiner_case()
{
	glBegin(GL_QUADS);

	glNormal3f(0.0f, 1.0f, 0.0f);
	
	glTexCoord2f(0.0f, 0.0f);
	glVertex3f(0.0f, 0.0f, 0.0f);
	
	glTexCoord2f(1.0f, 0.0f);
	glVertex3f(2.0f, 0.0f, 0.0f);
	
	glTexCoord2f(1.0f, 1.0f);
	glVertex3f(2.0f, 0.0f, 2.0f);
	
	glTexCoord2f(0.0f, 1.0f);
	glVertex3f(0.0f, 0.0f, 2.0f);
	
	
	glEnd();
}


void dessiner_sol()
{
	GLfloat nbT=5.0f;
	GLfloat y=-52.0f;
	GLfloat taille=-180.0f;
	glBegin(GL_QUADS);

	glNormal3f(0.0f, 1.0f, 0.0f);
	
	glTexCoord2f(0.0f, 0.0f);
	glVertex3f(-taille, y, -taille);
	
	glTexCoord2f(nbT, 0.0f);
	glVertex3f(taille, y, -taille);
	
	glTexCoord2f(nbT, nbT);
	glVertex3f(taille, y, taille);
	
	glTexCoord2f(0.0f, nbT);
	glVertex3f(-taille, y, taille);
	
	
	glEnd();
}


void dessiner_murs()
{
	GLfloat nbT=7.0f;
	GLfloat y=-52.0f;
	GLfloat taille=-180.0f;
	GLfloat plafond=-taille;
	
	
	
	glBegin(GL_QUADS);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(-taille, y, -taille);
		
		glTexCoord2f(nbT, 0.0f);
		glVertex3f(-taille, plafond, -taille);
		
		glTexCoord2f(nbT, nbT);
		glVertex3f(-taille, plafond, taille);
		
		glTexCoord2f(0.0f, nbT);
		glVertex3f(-taille, y, taille);
	glEnd();
	
	
	glBegin(GL_QUADS);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(taille, y, -taille);
			
		glTexCoord2f(nbT, 0.0f);
		glVertex3f(taille, plafond, -taille);
			
		glTexCoord2f(nbT, nbT);
		glVertex3f(taille, plafond, taille);
			
		glTexCoord2f(0.0f, nbT);
		glVertex3f(taille, y, taille);
	glEnd();
	
	
	glBegin(GL_QUADS);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(-taille, plafond, -taille);
			
		glTexCoord2f(nbT, 0.0f);
		glVertex3f(taille, plafond, -taille);
			
		glTexCoord2f(nbT, nbT);
		glVertex3f(taille, plafond, taille);
			
		glTexCoord2f(0.0f, nbT);
		glVertex3f(-taille, plafond, taille);
	glEnd();
	
	
	glBegin(GL_QUADS);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(-taille, plafond, taille);
					
		glTexCoord2f(nbT, 0.0f);
		glVertex3f(taille, plafond, taille);
					
		glTexCoord2f(nbT, nbT);
		glVertex3f(taille, y, taille);
					
		glTexCoord2f(0.0f, nbT);
		glVertex3f(-taille, y, taille);
	glEnd();	
	
	glBegin(GL_QUADS);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(-taille, plafond, -taille);
				
		glTexCoord2f(nbT, 0.0f);
		glVertex3f(taille, plafond, -taille);
				
		glTexCoord2f(nbT, nbT);
		glVertex3f(taille, y, -taille);
				
		glTexCoord2f(0.0f, nbT);
		glVertex3f(-taille, y, -taille);
	glEnd();
}


void dessiner_porte()
{
	GLfloat nbT=1.0f;
	GLfloat y=-52.0f;
	GLfloat taille=-180.0f;
	GLfloat plafond=-taille/2;
	
	glBegin(GL_QUADS);
		glTexCoord2f(0.0f, nbT);
		glVertex3f(taille+1.0f, y, -taille+(taille/10.0f));
				
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(taille+1.0f, plafond, -taille+(taille/10.0f));
				
		glTexCoord2f(nbT, 0.0f);
		glVertex3f(taille+1.0f, plafond, -taille+(taille/1.5f));
				
		glTexCoord2f(nbT, nbT);
		glVertex3f(taille+1.0f, y, -taille+(taille/1.5f));
	glEnd();
	
}


