/**
 * @file pieces.cpp
 * @author Frederic MARTIN [martin-frederic@users.sourceforge.net]
 * @brief Classe des pieces de l'échiquier
 */



#include "pieces.h"
#include "models.h"
#include <string>
#include <math.h>

// -------------------------------------------------------------------------- //
// --------------------------------- PPiece --------------------------------- //
// -------------------------------------------------------------------------- //

bool PPiece::peut_etre_pris_en_passant=false;
unsigned char PPiece::position_de_la_prise_en_passant=65;
unsigned char PPiece::position1_requise_pour_la_prise_en_passant=65;
unsigned char PPiece::position2_requise_pour_la_prise_en_passant=65;

PPiece::PPiece()
{
	for(int i=0;i<32;i++)
		tableau_des_possibilites_de_deplacement[i]=99;//n'importe quelle valeur supérieure à 64 fait l'affaire
	// par défaut une nouvelle pièce est une case vide
	couleur=CASE_VIDE;
	typeDePiece=VIDE;
	echiquier=NULL;
	anim=false;
}

PPiece::PPiece(int _typeDePiece,
			   GLuint _id_display_list, 
			   unsigned char _posX, 
			   unsigned char _posZ, 
			   unsigned char _couleur,
			   PPiece** _echiquier )
{
	typeDePiece=_typeDePiece;
	id_display_list=_id_display_list;
	posX=_posX;
	posZ=_posZ;
	couleur=_couleur;
	echiquier=_echiquier;
	anim=false;
}



PPiece::PPiece(PPiece& p)
{
	for(int i=0;i<32;i++)
		tableau_des_possibilites_de_deplacement[i]=p.tableau_des_possibilites_de_deplacement[i];
	
	typeDePiece=p.typeDePiece;
	couleur=p.couleur;
	id_display_list=p.id_display_list;
	posX=p.posX;
	posZ=p.posZ;
	echiquier=p.echiquier;
	anim=p.anim;
	px=p.px;
	py=p.py;
	pz=p.pz;
	t=p.t;
}

PPiece::~PPiece()
{
	
}




void PPiece::afficher()
{
	if (couleur==CASE_VIDE)
		return;
	
	GLfloat material_blanc_Ka[] = {0.2f,0.2f,0.2f,1.0f};
	GLfloat material_blanc_Kd[] = {0.9f,0.85f,0.55f,1.0f};
	GLfloat material_blanc_Ks[] = {0.7f,0.7f,0.7f,1.0f};
	GLfloat material_blanc_Ke[] = {0.0f,0.0f,0.0f,1.0f};
	
	GLfloat material_noir_Ka[] = {0.05f,0.05f,0.05f,1.0f};
	GLfloat material_noir_Kd[] = {0.1f,0.1f,0.1f,1.0f};
	GLfloat material_noir_Ks[] = {1.0f,1.0f,1.0f,1.0f};
	GLfloat material_noir_Ke[] = {0.0f,0.0f,0.0f,1.0f};
	
	glPushMatrix();
	
	if (anim)
	{
		if (t<=1540.0f)
			t+=pGlobalInfos->tempsEcoule();
		else
			anim=false;
		
		GLfloat pax=posX*2.0f;
		GLfloat paz=posZ*2.0f;
		
		if (typeDePiece==CAVALIER)
			glTranslatef(sin(t/1000.0f)*(pax-px)+px,py+5.0f*sin(t/500.0f),sin(t/1000.0f)*(paz-pz)+pz);
		else
			glTranslatef(sin(t/1000.0f)*(pax-px)+px,py,sin(t/1000.0f)*(paz-pz)+pz);
	}
	else
	{
		glTranslatef(posX*2.0f,0.02f,posZ*2.0f);
	}
	
	if (couleur==BLANC)
	{
		glMaterialfv(GL_FRONT,GL_AMBIENT,material_blanc_Ka);
		glMaterialfv(GL_FRONT,GL_DIFFUSE,material_blanc_Kd);
		glMaterialfv(GL_FRONT,GL_SPECULAR,material_blanc_Ks);
		glMaterialfv(GL_FRONT,GL_EMISSION,material_blanc_Ke);
		glMateriali(GL_FRONT,GL_SHININESS,128);
	}
	else if (couleur==NOIR)
	{
			
		glMaterialfv(GL_FRONT_AND_BACK,GL_AMBIENT,material_noir_Ka);
		glMaterialfv(GL_FRONT_AND_BACK,GL_DIFFUSE,material_noir_Kd);
		glMaterialfv(GL_FRONT_AND_BACK,GL_SPECULAR,material_noir_Ks);
		glMaterialfv(GL_FRONT_AND_BACK,GL_EMISSION,material_noir_Ke);
		glMateriali(GL_FRONT_AND_BACK,GL_SHININESS,15);
			
		glRotatef(180.0f,0.0f,1.0f,0.0f); // on la retourne pour qu'elle soit face aux blanches
	}
		
	glCallList(id_display_list);
	
	glPopMatrix();
}


void PPiece::deplacer(unsigned char _posX, unsigned char _posZ)
{
	/* 

	principe du déplacement d'une pièce :
	 
	on crée une nouvelle case vide
	on supprime la case d'arrivée
	on déplace la pièce depuis la case de départ jusqu'à la case d'arrivée
	on met la case vide à la case de départ de la pièce
	
	En dernier lieu on déclenche l'animation et on vérifie les coups spéciaux :
	- La promotion
	- La prise en passant
	- Le roque
	
	Ici la vitesse n'est pas cruciale puisque cette fonction n'est pas appelée par l'IA
	
	*/
	
	PPiece* case_vide;
	case_vide = new PPiece;
	delete echiquier[_posX+8*_posZ];
	echiquier[_posX+8*_posZ]=echiquier[posX+8*posZ];
	
	echiquier[posX+8*posZ]=case_vide;
	
	// on lance ensuite l'animation de la pièce avec les
	// bons paramètres
	anim=true;		// Active l'animation
	t=0.0f;			// Temps t pour l'équation paramétrique de l'animation
	px=posX*2.0f;	// coordonnées dans le repère OpenGL
	py=0.02f;		// coordonnées dans le repère OpenGL
	pz=posZ*2.0f;	// coordonnées dans le repère OpenGL
	
	
	// on oublie pas finalement de mettre à jour les coordonnées de la pièce
	unsigned char dX=posX;// sauvegarde la position de départ
	unsigned char dZ=posZ;// sauvegarde la position de départ
	posX=_posX;
	posZ=_posZ;
	
	// coups spéciaux : promotion et prise en passant
	if (typeDePiece==PION)
	{
		// on vérifie la promotion
		if ((posZ==0 && couleur==NOIR) || (posZ==7 && couleur==BLANC))
		{
			typeDePiece=REINE;
			id_display_list=(id_display_list-OBJ_PION)+OBJ_DAME;
		}
		
		// on vérifie la prise en passant	
		if (peut_etre_pris_en_passant)
		{
			// Si on bouge en diagonale et que derrière nous on a un pion
			// alors on supprime le pion : c'est la prise en passant
			if ((dX-posX)!=0 && (posX+8*posZ==position_de_la_prise_en_passant))
			{
				
				if (couleur==BLANC)
				{
					delete echiquier[posX+8*(posZ-1)];
					PPiece* nouvelle_case_vide=new PPiece;
					echiquier[posX+8*(posZ-1)]=nouvelle_case_vide;
				}
				if (couleur==NOIR)
				{
					delete echiquier[posX+8*(posZ+1)];
					PPiece* nouvelle_case_vide=new PPiece;
					echiquier[posX+8*(posZ+1)]=nouvelle_case_vide;
				}
			}
			peut_etre_pris_en_passant=false;
			position_de_la_prise_en_passant=65;
			position1_requise_pour_la_prise_en_passant=65;
			position2_requise_pour_la_prise_en_passant=65;
		}
							
		if (couleur==BLANC && dZ==1 && posZ==3)
		{
			peut_etre_pris_en_passant=true;
			position_de_la_prise_en_passant=posX+8*(posZ-1);
			position1_requise_pour_la_prise_en_passant=posX+1+8*posZ;
			position2_requise_pour_la_prise_en_passant=posX-1+8*posZ;
		}
		if (couleur==NOIR && dZ==6 && posZ==4)
		{
			peut_etre_pris_en_passant=true;
			position_de_la_prise_en_passant=posX+8*(posZ+1);
			position1_requise_pour_la_prise_en_passant=posX+1+8*posZ;
			position2_requise_pour_la_prise_en_passant=posX-1+8*posZ;
		}
		
		
	}
	else
	{
		position_de_la_prise_en_passant=65;
		position1_requise_pour_la_prise_en_passant=65;
		position2_requise_pour_la_prise_en_passant=65;
	}
	
	
	// on vérifie le roque
	if (typeDePiece==ROI)
	{
		PPiece* p;
		
		// échange la tour et la pièce vide pour faire le roque
		// petit roque
		if (dX-posX == 2)
		{
			p=echiquier[dX-3+8*dZ];
			echiquier[dX-3+8*dZ]=echiquier[dX-1+8*dZ];
			echiquier[dX-1+8*dZ]=p;
			p->posX+=2;
		}
		else
		//grand roque
		if (posX-dX == 2)
		{
			p=echiquier[dX+4+8*dZ];
			echiquier[dX+4+8*dZ]=echiquier[dX+1+8*dZ];
			echiquier[dX+1+8*dZ]=p;
			p->posX-=3;
		}
	}
	
}


void PPiece::deplacer_rapidement(unsigned char nouvellePosition)
{
	// déplacement optimisé pour l'IA
	// on ne fait aucun test, seulement le déplacement
	echiquier[nouvellePosition]=echiquier[posX+8*posZ];
	// on oublie pas finalement de mettre à jour les coordonnées de la pièce
	posX=nouvellePosition%8;
	posZ=nouvellePosition/8;
}


unsigned char* PPiece::possibilites_de_deplacement(bool IA)
{
	for(int i=0;i<32;i++)
		tableau_des_possibilites_de_deplacement[i]=99;//n'importe quelle valeur supérieure à 64 fait l'affaire

	switch (typeDePiece)
	{
		case VIDE:
			return tableau_des_possibilites_de_deplacement;
			break;
		case TOUR:
		{
			int i=0;
	
			// gauche
	
			int j=posX+1;
			unsigned char nouvellePosition=j+8*posZ;
	
			if (j<8)
			{
				while(echiquier[nouvellePosition]->couleur==CASE_VIDE)
				{
					tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
					j++;
					if (j==8)
						break;
					else
						nouvellePosition=j+8*posZ;
				}
				if (j<8)
					if (echiquier[nouvellePosition]->couleur!=couleur)
						tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
			}
	
			// droite
	
			j=posX-1;
			nouvellePosition=j+8*posZ;
	
			if (j>-1)
			{
				while(echiquier[nouvellePosition]->couleur==CASE_VIDE)
				{
					tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
					j--;
					if (j<0)
						break;
					else
						nouvellePosition=j+8*posZ;
				}
				if (j>-1)
					if (echiquier[nouvellePosition]->couleur!=couleur)
						tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
			}
	
			// bas
	
			j=posZ-1;
			nouvellePosition=posX+8*j;
	
			if (j>-1)
			{
				while(echiquier[nouvellePosition]->couleur==CASE_VIDE)
				{
					tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
					j--;
					if (j<0)
						break;
					else
						nouvellePosition=posX+8*j;
				}
				if (j>-1)
					if (echiquier[nouvellePosition]->couleur!=couleur)
						tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
			}
	
			// haut
	
			j=posZ+1;
			nouvellePosition=posX+8*j;
	
			if (j<8)
			{
				while(echiquier[nouvellePosition]->couleur==CASE_VIDE)
				{
					tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
					j++;
					if (j==8)
						break;
					else
						nouvellePosition=posX+8*j;
				}
				if (j<8)
					if (echiquier[nouvellePosition]->couleur!=couleur)
						tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
			}
	
	
			return tableau_des_possibilites_de_deplacement;
		}
		break;
		
		case PION:
		{
			int i=0;
	
	
			if (couleur==BLANC)
			{
				// cas où le pion est sur la deuxième ligne : il peut faire 2 cases en avant sauf s'il y a quelqu'un
				// sur une des cases de devant
				if (posZ==1)
				{
					unsigned char nouvellePosition=posX+8*(posZ+2);
					if (echiquier[nouvellePosition]->couleur==CASE_VIDE &&
									   echiquier[nouvellePosition-8]->couleur==CASE_VIDE)
						tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;// deux cases au dessus
				}
		
				// le pion peut avancer d'une case à condition que la case d'après soit vide
				if (posZ<7)
				{
					unsigned char nouvellePosition=posX+8*(posZ+1);
					if (echiquier[nouvellePosition]->couleur==CASE_VIDE)
						tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
				}
		
				// le pion peut prendre une pièce mais que sur la diagonale (ici gauche)
				if (posZ<7 && posX<7)
				{
					unsigned char nouvellePosition=(posX+1)+8*(posZ+1);
					if (echiquier[nouvellePosition]->couleur!=couleur&&
									   (echiquier[nouvellePosition]->couleur!=CASE_VIDE))
						tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
				}
		
				// le pion peut prendre une pièce mais que sur la diagonale (ici droite)
				if (posZ<7 && posX>0)
				{
					unsigned char nouvellePosition=(posX-1)+8*(posZ+1);
					if (echiquier[nouvellePosition]->couleur!=couleur&&
									   (echiquier[nouvellePosition]->couleur!=CASE_VIDE))
						tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
				}
			}
			else
			{
				// cas où le pion est sur la deuxième ligne : il peut faire 2 cases en avant sauf s'il y a quelqu'un
				// sur une des cases de devant
				if (posZ==6)
				{
					unsigned char nouvellePosition=posX+8*(posZ-2);
					if (echiquier[nouvellePosition]->couleur==CASE_VIDE &&
									   echiquier[nouvellePosition+8]->couleur==CASE_VIDE)
						tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;// deux cases au dessus
				}
		
				// le pion peut avancer d'une case à condition que la case d'après soit vide
				if (posZ>0)
				{
					unsigned char nouvellePosition=posX+8*(posZ-1);
					if (echiquier[nouvellePosition]->couleur==CASE_VIDE)
						tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
				}
		
				// le pion peut prendre une pièce mais que sur la diagonale (ici gauche)
				if (posZ>0 && posX<7)
				{
					unsigned char nouvellePosition=(posX+1)+8*(posZ-1);
					if (echiquier[nouvellePosition]->couleur!=couleur&&
									   (echiquier[nouvellePosition]->couleur!=CASE_VIDE))
						tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
				}
		
				// le pion peut prendre une pièce mais que sur la diagonale (ici droite)
				if (posZ>0 && posX>0)
				{
					unsigned char nouvellePosition=(posX-1)+8*(posZ-1);
					if (echiquier[nouvellePosition]->couleur!=couleur&&
									   (echiquier[nouvellePosition]->couleur!=CASE_VIDE))
						tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
				}			
			}
			
			if (IA==false)
			{
				if (peut_etre_pris_en_passant)
					if (posX+8*posZ==position1_requise_pour_la_prise_en_passant || posX+8*posZ==position2_requise_pour_la_prise_en_passant)
						tableau_des_possibilites_de_deplacement[i++]=position_de_la_prise_en_passant;
			}
			
			return tableau_des_possibilites_de_deplacement;
			
		}
		break;

		
		case CAVALIER:
		{
			int i=0;
	
			if (posZ<7 && posX>1)
			{
				unsigned char nouvellePosition=(posX-2)+8*(posZ+1);
				if (echiquier[nouvellePosition]->couleur!=couleur)
					tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
			}
	
			if (posZ>0 && posX>1)
			{
				unsigned char nouvellePosition=(posX-2)+8*(posZ-1);
				if (echiquier[nouvellePosition]->couleur!=couleur)
					tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
			}
	
			if (posZ<6 && posX>0)
			{
				unsigned char nouvellePosition=(posX-1)+8*(posZ+2);
				if (echiquier[nouvellePosition]->couleur!=couleur)
					tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
			}
	
			if (posZ<6 && posX<7)
			{
				unsigned char nouvellePosition=(posX+1)+8*(posZ+2);
				if (echiquier[nouvellePosition]->couleur!=couleur)
					tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
			}
	
			if (posZ<7 && posX<6)
			{
				unsigned char nouvellePosition=(posX+2)+8*(posZ+1);
				if (echiquier[nouvellePosition]->couleur!=couleur)
					tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
			}
	
			if (posZ>0 && posX<6)
			{
				unsigned char nouvellePosition=(posX+2)+8*(posZ-1);
				if (echiquier[nouvellePosition]->couleur!=couleur)
					tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
			}
	
			if (posZ>1 && posX<7)
			{
				unsigned char nouvellePosition=(posX+1)+8*(posZ-2);
				if (echiquier[nouvellePosition]->couleur!=couleur)
					tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
			}
	
			if (posZ>1 && posX>0)
			{
				unsigned char nouvellePosition=(posX-1)+8*(posZ-2);
				if (echiquier[nouvellePosition]->couleur!=couleur)
					tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
			}
	
			return tableau_des_possibilites_de_deplacement;
			
		}
		break;
		
		
		
		case FOU:
		{
			int i=0;
	
			// haut + gauche
	
			int j=posX+1;
			int k=posZ+1;
			unsigned char nouvellePosition=j+8*k;
	
			if (j<8 && k<8)
			{
				while(echiquier[nouvellePosition]->couleur==CASE_VIDE)
				{
					tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
					j++;
					k++;
					if (j==8 || k==8)
						break;
					else
						nouvellePosition=j+8*k;
				}
				if (j<8 && k<8)
					if (echiquier[nouvellePosition]->couleur!=couleur)
						tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
			}
	
	
			// bas + gauche
	
			j=posX+1;
			k=posZ-1;
			nouvellePosition=j+8*k;
	
			if (j<8 && k>-1)
			{
				while(echiquier[nouvellePosition]->couleur==CASE_VIDE)
				{
					tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
					j++;
					k--;
					if (j==8 || k==-1)
						break;
					else
						nouvellePosition=j+8*k;
				}
				if (j<8 && k>-1)
					if (echiquier[nouvellePosition]->couleur!=couleur)
						tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
			}
	
			// bas + droite
	
			j=posX-1;
			k=posZ-1;
			nouvellePosition=j+8*k;
	
			if (j>-1 && k>-1)
			{
				while(echiquier[nouvellePosition]->couleur==CASE_VIDE)
				{
					tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
					j--;
					k--;
					if (j==-1 || k==-1)
						break;
					else
						nouvellePosition=j+8*k;
				}
				if (j>-1 && k>-1)
					if (echiquier[nouvellePosition]->couleur!=couleur)
						tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
			}
	
			// haut + droite
	
			j=posX-1;
			k=posZ+1;
			nouvellePosition=j+8*k;
	
			if (j>-1 && k<8)
			{
				while(echiquier[nouvellePosition]->couleur==CASE_VIDE)
				{
					tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
					j--;
					k++;
					if (j==-1 || k==8)
						break;
					else
						nouvellePosition=j+8*k;
				}
				if (j>-1 && k<8)
					if (echiquier[nouvellePosition]->couleur!=couleur)
						tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
			}
	
	
			return tableau_des_possibilites_de_deplacement;
			
		}
		break;
		
		case REINE:
		{
			int i=0;
			int j,k;
			unsigned char nouvellePosition;
	

			// gauche
	
			j=posX+1;
			nouvellePosition=j+8*posZ;
	
			if (j<8)
			{
				while(echiquier[nouvellePosition]->couleur==CASE_VIDE)
				{
					tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
					j++;
					if (j==8)
						break;
					else
						nouvellePosition=j+8*posZ;
				}
				if (j<8)
					if (echiquier[nouvellePosition]->couleur!=couleur)
						tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
			}
	
			// droite
	
			j=posX-1;
			nouvellePosition=j+8*posZ;
	
			if (j>-1)
			{
				while(echiquier[nouvellePosition]->couleur==CASE_VIDE)
				{
					tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
					j--;
					if (j<0)
						break;
					else
						nouvellePosition=j+8*posZ;
				}
				if (j>-1)
					if (echiquier[nouvellePosition]->couleur!=couleur)
						tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
			}
	
			// bas
	
			j=posZ-1;
			nouvellePosition=posX+8*j;
	
			if (j>-1)
			{
				while(echiquier[nouvellePosition]->couleur==CASE_VIDE)
				{
					tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
					j--;
					if (j<0)
						break;
					else
						nouvellePosition=posX+8*j;
				}
				if (j>-1)
					if (echiquier[nouvellePosition]->couleur!=couleur)
						tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
			}
	
			// haut
	
			j=posZ+1;
			nouvellePosition=posX+8*j;
	
			if (j<8)
			{
				while(echiquier[nouvellePosition]->couleur==CASE_VIDE)
				{
					tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
					j++;
					if (j==8)
						break;
					else
						nouvellePosition=posX+8*j;
				}
				if (j<8)
					if (echiquier[nouvellePosition]->couleur!=couleur)
						tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
			}
	
			// haut + gauche
	
			j=posX+1;
			k=posZ+1;
			nouvellePosition=j+8*k;
	
			if (j<8 && k<8)
			{
				while(echiquier[nouvellePosition]->couleur==CASE_VIDE)
				{
					tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
					j++;
					k++;
					if (j==8 || k==8)
						break;
					else
						nouvellePosition=j+8*k;
				}
				if (j<8 && k<8)
					if (echiquier[nouvellePosition]->couleur!=couleur)
						tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
			}
	
	
			// bas + gauche
	
			j=posX+1;
			k=posZ-1;
			nouvellePosition=j+8*k;
	
			if (j<8 && k>-1)
			{
				while(echiquier[nouvellePosition]->couleur==CASE_VIDE)
				{
					tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
					j++;
					k--;
					if (j==8 || k==-1)
						break;
					else
						nouvellePosition=j+8*k;
				}
				if (j<8 && k>-1)
					if (echiquier[nouvellePosition]->couleur!=couleur)
						tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
			}
	
			// bas + droite
	
			j=posX-1;
			k=posZ-1;
			nouvellePosition=j+8*k;
	
			if (j>-1 && k>-1)
			{
				while(echiquier[nouvellePosition]->couleur==CASE_VIDE)
				{
					tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
					j--;
					k--;
					if (j==-1 || k==-1)
						break;
					else
						nouvellePosition=j+8*k;
				}
				if (j>-1 && k>-1)
					if (echiquier[nouvellePosition]->couleur!=couleur)
						tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
			}
	
			// haut + droite
	
			j=posX-1;
			k=posZ+1;
			nouvellePosition=j+8*k;
	
			if (j>-1 && k<8)
			{
				while(echiquier[nouvellePosition]->couleur==CASE_VIDE)
				{
					tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
					j--;
					k++;
					if (j==-1 || k==8)
						break;
					else
						nouvellePosition=j+8*k;
				}
				if (j>-1 && k<8)
					if (echiquier[nouvellePosition]->couleur!=couleur)
						tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
			}
	
			return tableau_des_possibilites_de_deplacement;
		}
		break;
		
		case ROI:
		{
			int i=0;
			unsigned char nouvellePosition;
	
			if (posZ<7)
			{
				nouvellePosition=posX+8*(posZ+1);
				if (echiquier[nouvellePosition]->couleur!=couleur)
					tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
			}
	
			if (posZ>0)
			{
				nouvellePosition=posX+8*(posZ-1);
				if (echiquier[nouvellePosition]->couleur!=couleur)
					tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
			}
	
			if (posX>0)
			{
				nouvellePosition=(posX-1)+8*posZ;
				if (echiquier[nouvellePosition]->couleur!=couleur)
					tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
			}
	
			if (posX<7)
			{
				nouvellePosition=(posX+1)+8*posZ;
				if (echiquier[nouvellePosition]->couleur!=couleur)
					tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
			}
	
			if (posZ>0 && posX>0)
			{
				nouvellePosition=(posX-1)+8*(posZ-1);
				if (echiquier[nouvellePosition]->couleur!=couleur)
					tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
			}
	
			if (posZ>0 && posX<7)
			{
				nouvellePosition=(posX+1)+8*(posZ-1);
				if (echiquier[nouvellePosition]->couleur!=couleur)
					tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
			}
	
			if (posZ<7 && posX>0)
			{
				nouvellePosition=(posX-1)+8*(posZ+1);
				if (echiquier[nouvellePosition]->couleur!=couleur)
					tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
			}
	
			if (posZ<7 && posX<7)
			{
				nouvellePosition=(posX+1)+8*(posZ+1);
				if (echiquier[nouvellePosition]->couleur!=couleur)
					tableau_des_possibilites_de_deplacement[i++]=nouvellePosition;
			}
			
			if (IA==false)
			{
				if (couleur==BLANC)
				{
					if (posX==3 && posZ==0)
					{
						// petit roque
						if (echiquier[0]->typeDePiece==TOUR && echiquier[1]->typeDePiece==VIDE && echiquier[2]->typeDePiece==VIDE)
							tableau_des_possibilites_de_deplacement[i++]=1;
						// grand roque
						if (echiquier[7]->typeDePiece==TOUR && echiquier[6]->typeDePiece==VIDE && echiquier[5]->typeDePiece==VIDE && echiquier[4]->typeDePiece==VIDE)
							tableau_des_possibilites_de_deplacement[i++]=5;
					}
					
				}
				else
				if (couleur==NOIR)
				{
					if (posX==3 && posZ==7)
					{
						// petit roque
						if (echiquier[56]->typeDePiece==TOUR && echiquier[57]->typeDePiece==VIDE && echiquier[58]->typeDePiece==VIDE)
							tableau_des_possibilites_de_deplacement[i++]=57;
						// grand roque
						if (echiquier[63]->typeDePiece==TOUR && echiquier[62]->typeDePiece==VIDE && echiquier[61]->typeDePiece==VIDE && echiquier[60]->typeDePiece==VIDE)
							tableau_des_possibilites_de_deplacement[i++]=61;
					}
					
				}
			}
				
	
			return tableau_des_possibilites_de_deplacement;
			
		}
		break;
	}
	
	return tableau_des_possibilites_de_deplacement;
	

}

