/**
 * @file pieces.h
 * @author Frederic MARTIN [martin-frederic@users.sourceforge.net]
 * @brief Classe des pieces de l'échiquier
 */


#ifndef ___PIECES_H___
#define ___PIECES_H___

#include "globalInfos.h"

/*	const INFINI représente la valeur donnée au roi.
	pas vraiment infini mais suffisant pour que si un roi meurt
	la partie soit considérée comme terminée quelque soit la
	configuration de l'échiquier.
*/
const int INFINI=900000;



enum couleur_de_chaque_piece
{
	NOIR,
	BLANC,
	CASE_VIDE
};

enum type_et_valeur_de_chaque_piece
{
	VIDE=0,
	PION=100,
	CAVALIER=300,
	FOU=330,
	TOUR=500,
	REINE=950,
	ROI=INFINI
};


class PPiece
{
	public:
		PPiece();
		PPiece( int _typeDePiece,
				GLuint _id_display_list, 
				unsigned char _posX, 
				unsigned char _posZ, 
				unsigned char _couleur,
				PPiece** _echiquier);
		PPiece(PPiece& p);
		virtual ~PPiece();
		unsigned char* possibilites_de_deplacement(bool IA);
		void deplacer(unsigned char _posX, unsigned char _posZ);
		void deplacer_rapidement(unsigned char nouvellePosition);
		void afficher();
		int get_valeur(){return typeDePiece;}
		unsigned char get_position() {return posX+8*posZ;}
		unsigned char get_couleur() {return couleur;}
		void setEchiquier(PPiece** _echiquier){echiquier=_echiquier;}

	protected:
		GLuint id_display_list;
		int typeDePiece;
		unsigned char couleur;
		unsigned char posX;
		unsigned char posZ;
		PPiece** echiquier;
		unsigned char tableau_des_possibilites_de_deplacement[32];
	
		// pour l'animation
		bool anim;
		GLfloat px,py,pz;
		GLfloat t;
		
		// pour les coups spéciaux
		static bool peut_etre_pris_en_passant;
		static unsigned char position_de_la_prise_en_passant;
		static unsigned char position1_requise_pour_la_prise_en_passant;
		static unsigned char position2_requise_pour_la_prise_en_passant;
		
};






#endif

