/**
 * @file scene_jeu_principal.cpp
 * @author Frederic MARTIN [martin-frederic@users.sourceforge.net]
 * @brief Implemente E_scene_jeu_principal. Voir scenes.h pour la liste de toutes les scenes
 */



#include "scene_jeu_principal.h"
#include "utils.h"
#include "models.h"
#include "raytracing.h"
#include "IA.h"
#include <math.h>


// Constante bien connue pi
#ifndef DEUX_FOIS_PI
#define DEUX_FOIS_PI 6.28319f
#endif

// Instance actuelle de la classe E_scene_menu_principal. Elle est utilisée par
// les fonctions de CALL-BACK car elles ne peuvent pas être membre de la classe
E_scene_jeu_principal* instance_jeu_principal=NULL; 


// matériau "gold" trouvée sur http://devernay.free.fr/cours/opengl/materials.html
const GLfloat material_plateau_Ka[] = {0.24725f,0.1995f,0.0745f,1.0f};
const GLfloat material_plateau_Kd[] = {0.75164f,0.60648f,0.22648f,1.0f};
const GLfloat material_plateau_Ks[] = {0.628281f,0.555802f,0.366065f,1.0f};
const GLfloat material_plateau_Ke[] = {0.0f,0.0f,0.0f,1.0f};
// matériau "green rubber" trouvée sur http://devernay.free.fr/cours/opengl/materials.html
const GLfloat material_table_Ka[] = {0.0f,0.05f,0.0f,1.0f};
const GLfloat material_table_Kd[] = {0.4f,0.5f,0.4f,1.0f};
const GLfloat material_table_Ks[] = {0.04f,0.7f,0.04f,1.0f};
const GLfloat material_table_Ke[] = {0.0f,0.0f,0.0f,1.0f};
// matériau de la fleche
const GLfloat material_fleche_Ka[] = {0.0f,1.0f,0.0f,1.0f};
const GLfloat material_fleche_Kd[] = {0.2f,1.0f,0.2f,1.0f};
const GLfloat material_fleche_Ks[] = {0.0f,1.0f,0.0f,1.0f};
const GLfloat material_fleche_Ke[] = {0.0f,0.0f,0.0f,1.0f};

//--------------------------------------------------//
//--------------------------------------------------//
//    ** fonctions de CALL-BACK  pour le GUI **     //
//--------------------------------------------------//
//--------------------------------------------------//
void sur_bouton_quitter_jeu_principal()
{
	instance_jeu_principal->set_confirmation_quitter();
}

void sur_bouton_camera_par_defaut()
{
	instance_jeu_principal->position_camera_par_defaut=true;
}

void sur_bouton_confirmer_quitter_le_jeu_principal()
{
	instance_jeu_principal->fin_du_jeu_principal=true;
}

void sur_bouton_annuler_quitter_le_jeu_principal()
{
	instance_jeu_principal->unset_confirmation_quitter();
}

void sur_bouton_forcer()
{
	instance_jeu_principal->IA_doit_finir_immediatement=true;
}


//--------------------------------------------------//
//--------------------------------------------------//
//         **  E_scene_jeu_principal **             //
//--------------------------------------------------//
//--------------------------------------------------//

E_scene_jeu_principal::E_scene_jeu_principal()
{
	textureSol						= NULL;
	damier							= NULL;
	fontTexture						= NULL;
	texture_bouton_quitter			= NULL;
	texture_bouton_camera			= NULL;
	textureMurs						= NULL;
	texturePorte					= NULL;
	instance_jeu_principal			= this;
}

E_scene_jeu_principal::~E_scene_jeu_principal()
{

}
		
		
void E_scene_jeu_principal::initialiser_echiquier()
{
	
	GLuint ROI_BLANC		= liste_des_modeles+OBJ_ROI;
	GLuint REINE_BLANC		= liste_des_modeles+OBJ_DAME;
	GLuint FOU_BLANC		= liste_des_modeles+OBJ_FOU;
	GLuint CAVALIER_BLANC	= liste_des_modeles+OBJ_CAVALIER;
	GLuint TOUR_BLANC		= liste_des_modeles+OBJ_TOUR;
	GLuint PION_BLANC		= liste_des_modeles+OBJ_PION;
	
	echiquier[0] = new PPiece(TOUR    ,TOUR_BLANC    ,0,0,BLANC,echiquier);
	echiquier[1] = new PPiece(CAVALIER,CAVALIER_BLANC,1,0,BLANC,echiquier);
	echiquier[2] = new PPiece(FOU     ,FOU_BLANC     ,2,0,BLANC,echiquier);
	echiquier[3] = new PPiece(ROI     ,ROI_BLANC     ,3,0,BLANC,echiquier);
	echiquier[4] = new PPiece(REINE   ,REINE_BLANC   ,4,0,BLANC,echiquier);
	echiquier[5] = new PPiece(FOU     ,FOU_BLANC     ,5,0,BLANC,echiquier);
	echiquier[6] = new PPiece(CAVALIER,CAVALIER_BLANC,6,0,BLANC,echiquier);
	echiquier[7] = new PPiece(TOUR    ,TOUR_BLANC    ,7,0,BLANC,echiquier);
	
	for(int i=8;i<16;i++)
		echiquier[i] = new PPiece(PION,PION_BLANC    ,i-8,1,BLANC,echiquier);
	
	for(int i=16;i<48;i++)
		echiquier[i] = new PPiece();
	
	echiquier[56] = new PPiece(TOUR    ,TOUR_BLANC    ,0,7,NOIR,echiquier);
	echiquier[57] = new PPiece(CAVALIER,CAVALIER_BLANC,1,7,NOIR,echiquier);
	echiquier[58] = new PPiece(FOU     ,FOU_BLANC     ,2,7,NOIR,echiquier);
	echiquier[59] = new PPiece(ROI     ,ROI_BLANC     ,3,7,NOIR,echiquier);
	echiquier[60] = new PPiece(REINE   ,REINE_BLANC   ,4,7,NOIR,echiquier);
	echiquier[61] = new PPiece(FOU     ,FOU_BLANC     ,5,7,NOIR,echiquier);
	echiquier[62] = new PPiece(CAVALIER,CAVALIER_BLANC,6,7,NOIR,echiquier);
	echiquier[63] = new PPiece(TOUR    ,TOUR_BLANC    ,7,7,NOIR,echiquier);
	
	for(int i=48;i<56;i++)
		echiquier[i] = new PPiece(PION,PION_BLANC    ,i-48,6,NOIR,echiquier);
				
						
}


void E_scene_jeu_principal::afficher_echiquier()
{
	glEnable(GL_TEXTURE_2D);
	glDisable(GL_LIGHTING);
	
	
	glColor3f(1.0f,1.0f,1.0f);
	damier->utiliser();
	glCallList(liste_des_modeles+OBJ_DAMIER);
	
	if (pGlobalInfos->get_sol_disco())
		glColor3f(0.7f+0.2f*sin(SDL_GetTicks()/150.0f),0.7f+0.3f*sin(SDL_GetTicks()/240.0f),0.7f+0.3f*sin(SDL_GetTicks()/100.0f));
	
	textureSol->utiliser();
	glCallList(liste_des_modeles+OBJ_SOL);
	
	glColor3f(1.0f,1.0f,1.0f);
	textureMurs->utiliser();
	glCallList(liste_des_modeles+OBJ_MURS);
	
	texturePorte->utiliser();
	glCallList(liste_des_modeles+OBJ_PORTE);
	
	glDisable(GL_TEXTURE_2D);
	glEnable(GL_LIGHTING);
	glDisable(GL_LIGHT1);
	
	const GLfloat lightPosition1    [] = { 7.0f, 10.0f, 7.0f, 0.0f };	// position de la source de lumière
	glLightfv(GL_LIGHT1, GL_POSITION, lightPosition1);
	
	// affichage du plateau
	glMaterialfv(GL_FRONT,GL_AMBIENT,material_plateau_Ka);
	glMaterialfv(GL_FRONT,GL_DIFFUSE,material_plateau_Kd);
	glMaterialfv(GL_FRONT,GL_SPECULAR,material_plateau_Ks);
	glMaterialfv(GL_FRONT,GL_EMISSION,material_plateau_Ke);
	glMaterialf(GL_FRONT,GL_SHININESS,51.2f);//51.2f = 128.0f*0.4f
		
	glPushMatrix();
	glTranslatef(7.0f,-0.1f,7.0f);
	glCallList(liste_des_modeles+OBJ_PLATEAU);
	glPopMatrix();
	
	// affichage de la table
	glMaterialfv(GL_FRONT,GL_AMBIENT,material_table_Ka);
	glMaterialfv(GL_FRONT,GL_DIFFUSE,material_table_Kd);
	glMaterialfv(GL_FRONT,GL_SPECULAR,material_table_Ks);
	glMaterialfv(GL_FRONT,GL_EMISSION,material_table_Ke);
	glMaterialf(GL_FRONT,GL_SHININESS,100.0f);//100.0f = 128.0f*0.78125ff
	
	glPushMatrix();
	glTranslatef(5.2f,-1.7f,7.1f);
	glCallList(liste_des_modeles+OBJ_TABLE);
	glPopMatrix();
	
	glEnable(GL_LIGHT1);
	
	
	for(int i=0;i<64;i++)
		echiquier[i]->afficher();
	
	
}



bool E_scene_jeu_principal::charger()
{
	if (deja_charge)
		return true;
	
	if (charger_objets_3d()==false)
	{
		printf("Loading a 3D model failed\n");
		return false;
	}
	
	textureSol = new Texture;
	if (false==textureSol->charger(E_DONNEES_TEXTURE_SOL,true))
	{
		printf("Loading of :"E_DONNEES_TEXTURE_SOL" failed\n");
		return false;
	}
	textureSol->SetTexParameter(GL_TEXTURE_WRAP_S,GL_REPEAT);
	textureSol->SetTexParameter(GL_TEXTURE_WRAP_T,GL_REPEAT);
	textureSol->SetTexParameter(GL_MODULATE,0);
	
	textureMurs = new Texture;
	if (false==textureMurs->charger(E_DONNEES_TEXTURE_MURS,true))
	{
		printf("Loading of :"E_DONNEES_TEXTURE_MURS" failed\n");
		return false;
	}
	textureMurs->SetTexParameter(GL_TEXTURE_WRAP_S,GL_REPEAT);
	textureMurs->SetTexParameter(GL_TEXTURE_WRAP_T,GL_REPEAT);
	textureMurs->SetTexParameter(GL_MODULATE,0);
	
	
	texturePorte = new Texture;
	if (false==texturePorte->charger(E_DONNEES_TEXTURE_PORTE,true))
	{
		printf("Loading of :"E_DONNEES_TEXTURE_PORTE" failed\n");
		return false;
	}
	texturePorte->SetTexParameter(GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE); 
	texturePorte->SetTexParameter(GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);
	
	
	damier = new Texture;
	if (false==damier->charger(E_DONNEES_TEXTURE_DAMIER,false))
	{
		printf("Loading of :"E_DONNEES_TEXTURE_DAMIER" failed\n");
		return false;
	}
	
	damier->SetTexParameter(GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE);
	damier->SetTexParameter(GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE);
	
	fontTexture = new Texture;
	if (false==fontTexture->charger(E_DONNEES_TEXTURE_FONT,false))
	{
		printf("Loading of :"E_DONNEES_TEXTURE_FONT" failed\n");
		return false;
	}
	
	texture_bouton_quitter = new Texture;
	if (false==texture_bouton_quitter->charger(E_DONNEES_TEXTURE_BOUTON_QUITTER,false))
	{
		printf("Loading of :"E_DONNEES_TEXTURE_BOUTON_QUITTER" failed\n");
		return false;
	}
	
	texture_bouton_camera = new Texture;
	if (false==texture_bouton_camera->charger(E_DONNEES_TEXTURE_BOUTON_CAMERA,false))
	{
		printf("Loading of :"E_DONNEES_TEXTURE_BOUTON_CAMERA" failed\n");
		return false;
	}
	
	texture_bouton_forcer = new Texture;
	if (false==texture_bouton_forcer->charger(E_DONNEES_TEXTURE_BOUTON_FORCE,false))
	{
		printf("Loading of :"E_DONNEES_TEXTURE_BOUTON_FORCE" failed\n");
		return false;
	}	
	
	
	un_gui.Init(1024.0f,768.0f, fontTexture->GetId() );

	
	bouton_camera_par_defaut.SetPos(930,5);
	bouton_camera_par_defaut.SetSize(32,32);
	bouton_camera_par_defaut.OnClick(sur_bouton_camera_par_defaut);
	bouton_camera_par_defaut.SetImage(texture_bouton_camera->GetId());
	un_gui.AddWidget(&bouton_camera_par_defaut);
	
	bouton_quitter.SetPos(980,5);
	bouton_quitter.SetSize(32,32);
	bouton_quitter.OnClick(sur_bouton_quitter_jeu_principal);
	bouton_quitter.SetImage(texture_bouton_quitter->GetId());
	un_gui.AddWidget(&bouton_quitter);
	
	bouton_confirmation_quitter.SetText("YES");
	bouton_confirmation_quitter.SetPos(300,330);
	bouton_confirmation_quitter.SetSize(150,30);
	bouton_confirmation_quitter.OnClick(sur_bouton_confirmer_quitter_le_jeu_principal);
	
	bouton_annuler_quitter.SetText("NO");
	bouton_annuler_quitter.SetPos(580,330);
	bouton_annuler_quitter.SetSize(150,30);
	bouton_annuler_quitter.OnClick(sur_bouton_annuler_quitter_le_jeu_principal);
	
	bouton_forcer.SetPos(880,5);
	bouton_forcer.SetSize(32,32);
	bouton_forcer.OnClick(sur_bouton_forcer);
	bouton_forcer.SetImage(texture_bouton_forcer->GetId());
	
	
	
	label_a_qui_de_jouer.SetColor(1.0f,1.0f,0.0f);
	label_a_qui_de_jouer.SetPos(10,5);
	un_gui.AddWidget(&label_a_qui_de_jouer);
	
	label_confirmation_quitter.SetText("Do you really want to quit now ?");
	label_confirmation_quitter.SetSize(1024,568);// tout l'écran ce qui centrera le message
	
	liste_des_modeles=genererla_liste_des_modeles();	// Crée les display lists des pièces
	
	glEnable(GL_LIGHTING); // activation de la lumière
	glEnable(GL_LIGHT0);// activation de la lumière 0
	glEnable(GL_LIGHT1);// activation de la lumière 1

	// définition des paramètres pour la lumière 0
	const GLfloat lightPosition0    [] = { 7.0f, 10.0f, 7.0f, 0.0f };	// position de la source de lumière
	const GLfloat lightColorAmbient [] = { 0.3f, 0.3f, 0.3f, 1.0f };	// couleur ambiante
	const GLfloat lightColorDiffuse [] = { 1.0f, 1.0f, 1.0f, 1.0f };	// couleur Diffuse
	const GLfloat lightColorSpecular[] = { 1.0f, 1.0f, 1.0f, 1.0f };	// couleur spéculaire	
	
	glLightfv(GL_LIGHT0, GL_POSITION, lightPosition0);
	glLightfv(GL_LIGHT0, GL_AMBIENT,  lightColorAmbient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE,  lightColorDiffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR, lightColorSpecular);
	
	glLightfv(GL_LIGHT1, GL_AMBIENT,  lightColorAmbient);
	glLightfv(GL_LIGHT1, GL_DIFFUSE,  lightColorDiffuse);
	glLightfv(GL_LIGHT1, GL_SPECULAR, lightColorSpecular);
	
	initialiser_echiquier();
	
	cameraX=0.0f;
	cameraY=0.0f;
	cameraZ=0.0f;

	fin_du_jeu_principal=false;
	
	sourisX=pGlobalInfos->SourisX();
	sourisY=pGlobalInfos->SourisY();
	
	caseXSelectionne=-1;
	caseZSelectionne=-1;
	
	couleur_choisie  = BLANC;
	couleur_en_cours = couleur_choisie;
	
	mode_de_jeu_choisi=pGlobalInfos->get_mode_de_jeu();
	IA_faible_priorite=pGlobalInfos->get_IA_faible_priorite();
	IA_difficulte=pGlobalInfos->get_IA_difficulte();
	IA_temps=pGlobalInfos->get_limite_de_temps_pour_IA();
	IA_ID=pGlobalInfos->get_IA_ID();
	
	if (mode_de_jeu_choisi==pGlobalInfos->JOUEUR_CONTRE_JOUEUR)
		if (couleur_choisie==BLANC)
			label_a_qui_de_jouer.SetText("Whites can begin");
		else
			label_a_qui_de_jouer.SetText("Blacks can begin");
	else
		if (couleur_choisie==BLANC)
			label_a_qui_de_jouer.SetText("You are white - You can begin");
		else
			label_a_qui_de_jouer.SetText("You are blacks - You can begin");
		
	label_a_qui_de_jouer.SetSize(30,20);
	
	doit_on_afficher_fenetre_de_confirmation_pour_quitter=false;
					
	joueur_blanc_en_echec=false;
	joueur_noir_en_echec=false;		
	partie_finie=false;
	IA_abandonne=false;
		
	position_camera_par_defaut=true;
	angleCameraZ_defaut=0.8f;
	cameraDist_defaut=23.0f;

	if (couleur_choisie==NOIR)
		angleCameraX_defaut=1.57f;
	if (couleur_choisie==BLANC)
		angleCameraX_defaut=-1.57f;
	
	
	angleCameraZ=angleCameraZ_defaut;
	angleCameraX=angleCameraX_defaut;
	cameraDist=cameraDist_defaut;
	
	largeur_ecran=(GLfloat)pGlobalInfos->largeurEcran();
	hauteur_ecran=(GLfloat)pGlobalInfos->hauteurEcran();
	
	tableau_des_possibilites_de_deplacement=NULL;
	IntelligenceArtificielle=NULL;
	IA_doit_finir_immediatement=false;
	IA_vient_de_finir_de_jouer=false;
			
	fin_du_chargement();
	return true;
}

void E_scene_jeu_principal::decharger()
{
	if (deja_charge==false)
		return;
	
	if (IntelligenceArtificielle)
	{
		int status;
		IA_doit_finir_immediatement=true;
		SDL_WaitThread(IntelligenceArtificielle,&status);
	}
	
	decharger_objets_3d();
	
	delete damier;
	damier=NULL;
	
	delete textureSol;
	textureSol=NULL;
	
	delete textureMurs;
	textureMurs=NULL;
	
	for(int i=0;i<64;i++)
	{
		delete echiquier[i];
		echiquier[i]=NULL;
	}
	
	delete fontTexture;
	fontTexture=NULL;
	
	delete texture_bouton_quitter;
	texture_bouton_quitter=NULL;
	
	delete texture_bouton_camera;
	texture_bouton_camera=NULL;
	
	delete texturePorte;
	texturePorte=NULL;
	
	delete texture_bouton_camera;
	texture_bouton_camera=NULL;
	
	un_gui.Destroy();
	
	deja_charge=false;
}


void E_scene_jeu_principal::actualiser()
{
	if (deja_charge==false)
	{
		if ( charger()==false )
		{
			printf("Loading of 'E_scene_jeu_principal' failed: exiting...\n");
			pGlobalInfos->quitter_le_jeu();
			return;
		}
	}
	
	int sourisX=(int) (1024.0f*pGlobalInfos->SourisX()/largeur_ecran);
	int sourisY=(int) (768.0f*pGlobalInfos->SourisY()/hauteur_ecran);
	
	actualiser_camera();
	afficher_echiquier();
	
	
	if (doit_on_afficher_fenetre_de_confirmation_pour_quitter)
	{
		afficher_fenetre_de_confirmation_pour_quitter();
		
		if (pGlobalInfos->touche_est_enfonce(SDLK_ESCAPE))
		{
			pGlobalInfos->relacher_touches();
			unset_confirmation_quitter();
		}
	}
	else
	{
		
		clique_sur_l_echiquier(pGlobalInfos->SourisX(), pGlobalInfos->SourisY());
		// la touche echap permet de désélectionner une pièce
		// si aucune pièce n'était séelectionné on quitte le jeu principal
		if (pGlobalInfos->touche_est_enfonce(SDLK_ESCAPE))
		{
			pGlobalInfos->relacher_touches();
			
			if (caseXSelectionne>-1 && caseZSelectionne>-1)
			{
				caseXSelectionne=-1;
				caseZSelectionne=-1;
			}
			else
				set_confirmation_quitter();
		}
	}
	
	afficher_barre_de_menu(sourisX, sourisY);
	
	if (fin_du_jeu_principal)
	{
		effaceEcran();
		decharger();
		pGlobalInfos->changer_de_scene(E_SCENE_MENU_PRINCIPAL);
		return;
	}
	
	
	if (IA_vient_de_finir_de_jouer)
	{
		IA_vient_de_finir_de_jouer=false;
		IA_doit_finir_immediatement=false;
		
		if (IA_abandonne || echec())
		{
			partie_finie=true;
			couleur_vainqueur=BLANC;
			label_a_qui_de_jouer.SetText("GAME OVER");
			return;
		}
		
		if (couleur_en_cours == BLANC)
			couleur_en_cours=NOIR;
		else
			couleur_en_cours=BLANC;

		// Vérification de l'échec
		verifier_echec_apres_coup();
		if (IA_ID)
			un_gui.RemoveWidget(&bouton_forcer);
	}
	

	SDL_Delay(5);
}



void E_scene_jeu_principal::actualiser_camera()
{
	Uint32 dt=pGlobalInfos->tempsEcoule();
	
	
	if (position_camera_par_defaut)
	{
		const GLfloat epsilon=0.035f;
		// si la machine est très lente alors dt risque d'être trop
		// élevé et les conditions pour sortir du mode de replacement
		// par défaut de la caméra ne seront pas rempli ce qui
		// bloquera définitivement la caméra. En forçant une valeur maximum
		// pour dt on évite ce problème.
		if (dt>20)
			dt=20;

		if (angleCameraX<angleCameraX_defaut)
			angleCameraX+=dt/600.0f;
		if (angleCameraX>angleCameraX_defaut)
			angleCameraX-=dt/600.0f;
		
		if (angleCameraZ<angleCameraZ_defaut)
			angleCameraZ+=dt/600.0f;
		if (angleCameraZ>angleCameraZ_defaut)
			angleCameraZ-=dt/600.0f;
		
		if (cameraDist<cameraDist_defaut)
			cameraDist+=dt/60.0f;
		if (cameraDist>cameraDist_defaut)
			cameraDist-=dt/60.0f;
		
		
		if (
			(angleCameraX_defaut-epsilon)<angleCameraX &&
			angleCameraX<(angleCameraX_defaut+epsilon) && 
			(angleCameraZ_defaut-epsilon)<angleCameraZ &&
			angleCameraZ<(angleCameraZ_defaut+epsilon) &&
			(cameraDist_defaut-0.5f)<cameraDist &&
			cameraDist<(cameraDist_defaut+0.5f) )
		{
			position_camera_par_defaut=false;
			angleCameraZ=angleCameraZ_defaut;
			angleCameraX=angleCameraX_defaut;
			cameraDist=cameraDist_defaut;			
		}
	}
	else
	{
			
		int mouseX=pGlobalInfos->SourisX();
		int mouseY=pGlobalInfos->SourisY();
		
		if (pGlobalInfos->touche_est_enfonce(SDLK_DOWN))
			angleCameraZ-=0.001f*dt;
		if (pGlobalInfos->touche_est_enfonce(SDLK_UP))
			angleCameraZ+=0.001f*dt;
		
		if (pGlobalInfos->touche_est_enfonce(SDLK_RIGHT))
			angleCameraX-=0.001f*dt;
		if (pGlobalInfos->touche_est_enfonce(SDLK_LEFT))
			angleCameraX+=0.001f*dt;
		
		if (pGlobalInfos->touche_est_enfonce(SDLK_PAGEUP))
			cameraDist-=dt/15.0f;
		if (pGlobalInfos->touche_est_enfonce(SDLK_PAGEDOWN))
			cameraDist+=dt/15.0f;
		
		
		
		if (pGlobalInfos->cliqueDroitEnfonce())
		{
			angleCameraX+=0.001f*dt*(mouseX-sourisX)/5.0f;
			angleCameraZ+=0.001f*dt*(mouseY-sourisY)/5.0f;
		}
		// pour le déplacement de la caméra à l'aide du clique droit de la souris
		sourisX=mouseX;
		sourisY=mouseY;
		
		if (pGlobalInfos->surMoletteHaut())
			cameraDist-=dt/6.0f;
		if (pGlobalInfos->surMoletteBas())
			cameraDist+=dt/6.0f;
		
		if (angleCameraX>DEUX_FOIS_PI)
			angleCameraX-=DEUX_FOIS_PI;
		if (angleCameraX<-DEUX_FOIS_PI)
			angleCameraX+=DEUX_FOIS_PI;
		if (angleCameraZ>1.57f) //un peu moins que Pi/2
			angleCameraZ=1.57f; //un peu moins que Pi/2
		if (angleCameraZ<-0.7f)
			angleCameraZ=-0.7f;
		if (cameraDist>80.0f)
			cameraDist=80.0f;
		if (cameraDist<23.0f)
			cameraDist=23.0f;
	
	}

	
	cameraX=cos(angleCameraX)*cos(angleCameraZ)*cameraDist+7.0f;
	cameraY=sin(angleCameraZ)*cameraDist;
	cameraZ=sin(angleCameraX)*cos(angleCameraZ)*cameraDist+7.0f;
	
	
	// Positionne la caméra
	gluLookAt(cameraX, cameraY, cameraZ,	// coordonnées de la position de la caméra
			  7.0f, 0.0f, 7.0f,			// coordonnées du centre de l'échiquier
			  0.0f, 1.0f, 0.0f);			// caméra tournée selon les y
}



void E_scene_jeu_principal::clique_sur_l_echiquier(int mouseX, int mouseY)
{
	// si ce n'est pas au tour du joueur de jouer alors on ne gère pas le clique sur l'échiquier
	if ((mode_de_jeu_choisi == pGlobalInfos->JOUEUR_CONTRE_IA) && (couleur_en_cours != couleur_choisie))
		return;

	if (pGlobalInfos->surCliqueGauche() && mouseY>=30)
	{
		double Vertices[12]={-1.0,0.0,-1.0,  15.0,0.0,-1.0,  15.0,0.0,15.0,  -1.0,0.0,15.0};//les coordonnées des sommets du plateau
		double Intersec[3]={0};// le point 3D sélectionné par l'utilisateur sera enregsitré dans ce tableau
	
		if (angleCameraZ>0.01f && 
				  SelectPolygon(cameraX, cameraY, cameraZ, Vertices, Intersec, mouseX, mouseY))
		{
			// si le joueur a déjà sélectionné une pièce cela veut dire qu'il veut la jouer
			if (caseXSelectionne>-1 && caseZSelectionne>-1)
			{
				int caseXFuture=(int)((Intersec[0]+1.0)/2.0);
				int caseZFuture=(int)((Intersec[2]+1.0)/2.0);
				// ne devrait jamais arriver mais on teste au cas où
				if (caseXFuture<0)
					caseXFuture=0;
				if (caseXFuture>7)
					caseXFuture=7;
				if (caseZFuture<0)
					caseZFuture=0;
				if (caseZFuture>7)
					caseZFuture=7;
				// on recherche maintenant si la case sur laquelle il vient de cliquer fait partie
				// des possibilités de déplacement
				int i=0;
				while (tableau_des_possibilites_de_deplacement[i]<65)
				{
					int caseXPossible=tableau_des_possibilites_de_deplacement[i]%8;
					int caseZPossible=tableau_des_possibilites_de_deplacement[i]/8;

					if (caseXPossible==caseXFuture && caseZPossible==caseZFuture)
					{
						int indice_piece=caseXSelectionne+8*caseZSelectionne;
						echiquier[indice_piece]->deplacer(caseXFuture,caseZFuture);
						
						caseXSelectionne=-1;
						caseZSelectionne=-1;
						if (couleur_en_cours == BLANC)
							couleur_en_cours = NOIR;
						else
							couleur_en_cours = BLANC;
						
						
						// Vérification de l'échec
						verifier_echec_apres_coup();
						
						if (partie_finie)
							return;
								
						
						// ------- MODE JOUEUR_CONTRE_JOUEUR -------
						if (mode_de_jeu_choisi == pGlobalInfos->JOUEUR_CONTRE_JOUEUR)
						{
							if (couleur_en_cours==BLANC)
								angleCameraX_defaut=-1.57f;
							else
								angleCameraX_defaut=1.57f;
							
							position_camera_par_defaut=true;// on remet la caméra devant le camp qui va jouer
						}
						
						
						
						// ------- MODE JOUEUR_CONTRE_IA -------
						if (mode_de_jeu_choisi == pGlobalInfos->JOUEUR_CONTRE_IA)
						{
							if (IA_ID)
								un_gui.AddWidget(&bouton_forcer);
							
							label_a_qui_de_jouer.SetText("I am thinking now...");
							label_a_qui_de_jouer.SetSize(30,20);
							
							thread_data[0]=echiquier;
							thread_data[1]=&couleur_en_cours;
							thread_data[2]=&IA_doit_finir_immediatement;
							thread_data[3]=&IA_vient_de_finir_de_jouer;
							thread_data[4]=&IA_faible_priorite;
							thread_data[5]=&IA_difficulte;
							thread_data[6]=&IA_temps;
							thread_data[7]=&IA_ID;
							thread_data[8]=&IA_abandonne;
							IntelligenceArtificielle=SDL_CreateThread(IAJouer,thread_data);
						}
						return;
					}
					i++;
				}
			}

			caseXSelectionne=(int)((Intersec[0]+1.0f)/2.0f);
			caseZSelectionne=(int)((Intersec[2]+1.0f)/2.0f);
			// ne devrait jamais arriver mais on teste au cas où
			if (caseXSelectionne<0)
				caseXSelectionne=0;
			if (caseXSelectionne>7)
				caseXSelectionne=7;
			if (caseZSelectionne<0)
				caseZSelectionne=0;
			if (caseZSelectionne>7)
				caseZSelectionne=7;
			
			// vérifie que la pièce qui est sur la case sélectionnée est bien de la couleur du camp choisie
			// si ce n'est pas le cas on désélectionne la case
			int indice_piece=caseXSelectionne+8*caseZSelectionne;
			if (echiquier[indice_piece]->get_couleur()!=couleur_en_cours)
			{
				caseXSelectionne=-1;
				caseZSelectionne=-1;
			}
			else
			{
				tableau_des_possibilites_de_deplacement=echiquier[indice_piece]->possibilites_de_deplacement(false);
				verifier_tableau_des_possibilites_de_deplacement_en_cas_d_echec(indice_piece);
			}
		}
	}
	
	if (caseXSelectionne>-1 && caseZSelectionne>-1)
	{
		glDisable(GL_TEXTURE_2D);
		glDisable(GL_LIGHTING);
		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		glEnable(GL_BLEND);
		
		// affiche la case de la pièce sélectionné en bleu
		glColor4f(0.0f,0.0f,0.9f,0.5f);
		glPushMatrix();
		glTranslatef(-1.0f+2.0f*caseXSelectionne,0.01f,-1.0f+2.0f*caseZSelectionne);// positionne le sélecteur
		glCallList(liste_des_modeles+OBJ_CASE);//affiche le sélecteur
		if (pGlobalInfos->get_fleche_de_selection())
			afficher_fleche_de_selection();// affiche la flèche en 3d au dessus de la pièce sélectionnée
		glPopMatrix();

		// affiche les possibilités de deplacement pour le joueur
		int i=0;
		while (tableau_des_possibilites_de_deplacement[i]<65)
		{
			int caseXPossible=tableau_des_possibilites_de_deplacement[i]%8;
			int caseZPossible=tableau_des_possibilites_de_deplacement[i]/8;
			unsigned char couleur_de_la_case_possible;
			unsigned char couleur_de_la_case_selectionne;
			
			couleur_de_la_case_selectionne=echiquier[caseXSelectionne+8*caseZSelectionne]->get_couleur();
			couleur_de_la_case_possible=echiquier[tableau_des_possibilites_de_deplacement[i]]->get_couleur();
			
			GLfloat transparence_de_la_case_arrivee=0.4f+0.2*sin(SDL_GetTicks()/300.0f);
			
			if (couleur_de_la_case_possible!=couleur_de_la_case_selectionne && couleur_de_la_case_possible!=CASE_VIDE)
				glColor4f(0.9f,0.0f,0.0f,transparence_de_la_case_arrivee);
			else
				glColor4f(0.0f,0.9f,0.0f,transparence_de_la_case_arrivee);
			glPushMatrix();
			glTranslatef(-1.0f+2.0f*caseXPossible,0.01f,-1.0f+2.0f*caseZPossible);// positionne le sélecteur
			glCallList(liste_des_modeles+OBJ_CASE);//affiche le sélecteur
			glPopMatrix();
			
			i++;
		}
		glEnable(GL_LIGHTING);
	}
}






void E_scene_jeu_principal::afficher_barre_de_menu(int mouseX, int mouseY)
{
	aller_en_2d();

	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_BLEND);
	glDisable(GL_TEXTURE_2D);
	
	// affiche un rectangle transparent sur les 30 premières lignes
//	glColor4f(0.0f,0.0f,0.0f,0.5f);
//	dessiner_rectangle_plein(0,0,1024,30);
	
	un_gui.Update(mouseX,mouseY,pGlobalInfos->cliqueGaucheEnfonce());


	quitter_la_2d();
	
}






void E_scene_jeu_principal::afficher_fleche_de_selection()
{
	static GLfloat y_rot=0.0f;
	y_rot+=pGlobalInfos->tempsEcoule()/10.0f;
	
	glEnable(GL_LIGHTING);
	
	glPushMatrix();
	
	glScalef(0.4f,0.4f,0.4f);
	glTranslatef(2.5f,13.5f+sin(SDL_GetTicks()/120.0f),2.5f);
	glRotatef(180.0f,1.0f,0.0f,0.0f);
	glRotatef(y_rot,0.0f,1.0f,0.0f);
	
	glMaterialfv(GL_FRONT,GL_AMBIENT,material_fleche_Ka);
	glMaterialfv(GL_FRONT,GL_DIFFUSE,material_fleche_Kd);
	glMaterialfv(GL_FRONT,GL_SPECULAR,material_fleche_Ks);
	glMaterialfv(GL_FRONT,GL_EMISSION,material_fleche_Ke);
	glMateriali(GL_FRONT,GL_SHININESS,128);
	
	glCallList(liste_des_modeles+OBJ_FLECHE);
	glPopMatrix();
	
	glDisable(GL_LIGHTING);
	
}

void E_scene_jeu_principal::set_confirmation_quitter()
{
	if (doit_on_afficher_fenetre_de_confirmation_pour_quitter)
		return;
	
	un_gui.AddWidget(&bouton_confirmation_quitter);
	un_gui.AddWidget(&bouton_annuler_quitter);
	un_gui.AddWidget(&label_confirmation_quitter);

	doit_on_afficher_fenetre_de_confirmation_pour_quitter=true;
}
		

void E_scene_jeu_principal::unset_confirmation_quitter()
{
	un_gui.RemoveWidget(&bouton_confirmation_quitter);
	un_gui.RemoveWidget(&bouton_annuler_quitter);
	un_gui.RemoveWidget(&label_confirmation_quitter);

	doit_on_afficher_fenetre_de_confirmation_pour_quitter=false;
}


void E_scene_jeu_principal::afficher_fenetre_de_confirmation_pour_quitter()
{
	aller_en_2d();

	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_BLEND);
	glDisable(GL_TEXTURE_2D);
	
	// affiche un rectangle transparent 
	glColor4f(0.0f,0.0f,0.0f,0.75f);
	dessiner_rectangle_plein(240,260,784,400);
	
	quitter_la_2d();
}

/*
	Cette fonction retourne true si le joueur de la couleur_en_cours est en échec
	Pour savoir s'il y a effectivement echec, la fonction prend chaque pièce du camp
	ennemi et regarde si une d'entre elle peut aller jusqu'au roi. Si elle le peut
	c'est qu'il y a bien echec, sinon la fonction renvoie false.
*/
bool E_scene_jeu_principal::echec()
{
	unsigned char* pdp=NULL;//pour récupérer les déplacements possibles
	unsigned char possibilites_de_deplacement[32];// pour sauver les déplacements possibles
	int j;// j est initialisée un peu plus bas, pas de soucis


	// pour chacune des pièces du jeu
	for(int i=0;i<64;i++)
	{
		// si on a sélectionné une pièce qui nous appartient on passe à la suivante
		if (echiquier[i]->get_couleur()==couleur_en_cours)
			continue;
		
		// on récupère ses possibilités de déplacement pour la pièce sélectionnée
		pdp=echiquier[i]->possibilites_de_deplacement(true);
		for(j=0;j<32;j++)
			possibilites_de_deplacement[j]=pdp[j];
		
		j=0;
		// Tant qu'il y a un déplacement possible
		while (possibilites_de_deplacement[j]<65)
		{
			// si on peut accéder au ROI (ça sera un roi ennemi car on ne peut pas
			// se déplacer sur une de nos propres pièces) alors il y a échec
			if (echiquier[possibilites_de_deplacement[j]]->get_valeur()==ROI)
				return true;
			
			j++;
		}
	}
	
	// aucune pièce n'a pu accéder au roi ennemi donc il n'y a pas échec
	return false;
	
	
}


bool E_scene_jeu_principal::echec_et_mat()
{
	for(int i=0;i<64;i++)
	{
		// si on a sélectionné une pièce qui ne nous appartient pas on passe à la suivante
		if (echiquier[i]->get_couleur()!=couleur_en_cours)
			continue;
		
		// on regarde les coups qu'on peut faire avec cette pièce
		verifier_tableau_des_possibilites_de_deplacement_en_cas_d_echec(i);
		
		// si on peut en faire ne serait-ce qu'un seul on sait qu'il n'y a pas echec et mat
		// donc on a terminé
		if (tableau_des_possibilites_de_deplacement[0]<65)
			return false;
	}
	
	// si on arrive ici c'est qu'on a tout essayé sans succès et qu'il n'y a plus
	// aucun coup possible donc il y a echec et mat ! (BRAVO au vaincqueur !)
	return true;
}



void E_scene_jeu_principal::verifier_tableau_des_possibilites_de_deplacement_en_cas_d_echec(int indice_piece)
{
	unsigned char* pdp=NULL;//pour récupérer les déplacements possibles
	unsigned char possibilites_de_deplacement[32];// pour sauver les déplacements possibles
	unsigned char nouvelles_possibilites_de_deplacement[32];// pour sauver les nouveaux déplacements possibles
	int j=0;
	int k=0;
	PPiece piece_vide;
	PPiece* piece;
	
	for(int i=0;i<32;i++)
		nouvelles_possibilites_de_deplacement[i]=99;
	
	
	int i=indice_piece;
	
	// on récupère ses possibilités de déplacement pour la pièce sélectionnée
	pdp=echiquier[i]->possibilites_de_deplacement(false);
	for(j=0;j<32;j++)
		possibilites_de_deplacement[j]=pdp[j];
		
	j=0;
	
	// Tant qu'il y a un déplacement possible
	while (possibilites_de_deplacement[j]<65)
	{
		/*	on va déplacer notre pièce mais si à l'endroit du déplacement
		il y a une autre pièce on fait attention de s'en rappeler pour
		qu'au moment où on voudra "défaire" le coup on puisse la remettre
		*/
		piece=echiquier[possibilites_de_deplacement[j]];
		echiquier[i]->deplacer_rapidement(possibilites_de_deplacement[j]);
		echiquier[i]=&piece_vide;

		// si ce coup permet d'éviter l'echec on le sauve
		if (!echec())
		{
			nouvelles_possibilites_de_deplacement[k]=possibilites_de_deplacement[j];
			k++;
		}
		
		// on "défait" le coup qui vient d'être joué
		echiquier[possibilites_de_deplacement[j]]->deplacer_rapidement(i);
		// on remet la pièce qui avait été là au début ie: avant le déplacement
		echiquier[possibilites_de_deplacement[j]]=piece;
			
		
		j++;
	}

	
	// on sauvegarde la nouvelle liste des coups possibles dont chaque coup
	// permettra de jouer sans perdre le roi
	for(int i=0;i<32;i++)
		tableau_des_possibilites_de_deplacement[i]=nouvelles_possibilites_de_deplacement[i];
}


void E_scene_jeu_principal::verifier_echec_apres_coup()
{
	if (echec())
	{
		if (couleur_en_cours==BLANC)
		{
			joueur_blanc_en_echec=true;
			if (echec_et_mat())
			{
				partie_finie=true;
				couleur_vainqueur=NOIR;
				label_a_qui_de_jouer.SetText("ChessMate : BLACKS VICTORY !");
			}
			else
				label_a_qui_de_jouer.SetText("You are white - CHESS !");
		}
		else	// couleur_en_cours==NOIR
		{ 
			joueur_noir_en_echec=true;
			if (echec_et_mat())
			{
				partie_finie=true;
				couleur_vainqueur=BLANC;
				label_a_qui_de_jouer.SetText("ChessMate : WHITE VICTORY !");
			}
			else
				label_a_qui_de_jouer.SetText("You are black - CHESS !");
		}
	}
	else
	{
		joueur_blanc_en_echec=false;
		joueur_noir_en_echec=false;
		if (couleur_en_cours==BLANC)
			label_a_qui_de_jouer.SetText("You are white - It's your turn to play");
		else
			label_a_qui_de_jouer.SetText("You are black - It's your turn to play");
	}
		
	label_a_qui_de_jouer.SetSize(30,20);
}


