/**
 * @file scene_jeu_principal.h
 * @author Frederic MARTIN [martin-frederic@users.sourceforge.net]
 * @brief Implemente E_scene_jeu_principal. Voir scenes.h pour la liste de toutes les scenes
 */


#ifndef __E_SCENE_JEU_PRINCIPAL_H__
#define __E_SCENE_JEU_PRINCIPAL_H__


#include "scenes.h"
#include "pieces.h"
#include <SDL/SDL_thread.h>

class E_scene_jeu_principal: public Echec_Scene
{
	public:
		E_scene_jeu_principal();
		virtual ~E_scene_jeu_principal();
		virtual bool charger();
		virtual void decharger();
		virtual void actualiser();
		
		void set_confirmation_quitter();
		void unset_confirmation_quitter();
		
		// les attributs ci-dessous sont en publiques car certaines
		// fonctions non-membres (fonctions qui sont appelés lors du clique sur
		// un bouton du GUI par exemple) doivent pouvoir y accéder.
		bool fin_du_jeu_principal;
		bool position_camera_par_defaut;
		bool IA_doit_finir_immediatement;
		
	private:
		GLuint liste_des_modeles;
		PPiece* echiquier[64];// l'échiquier est un tableau à deux dimensions de 8*8 cases
		
		int sourisX;
		int sourisY;
		
		int caseXSelectionne;
		int caseZSelectionne;
		
		GLfloat cameraX, cameraY, cameraZ; // position de la caméra
		GLfloat angleCameraX, angleCameraZ, cameraDist;
		GLfloat angleCameraX_defaut,angleCameraZ_defaut,cameraDist_defaut;
		
		Texture* textureSol;
		Texture* damier;
		Texture* fontTexture;
		Texture* texture_bouton_quitter;
		Texture* texture_bouton_camera;
		Texture* textureMurs;
		Texture* texturePorte;
		Texture* texture_bouton_forcer;
		
		CGui un_gui;
		GLfloat largeur_ecran;
		GLfloat hauteur_ecran;
		
		
		CGuiButton bouton_afficher_historique;
		CGuiButton bouton_confirmation_quitter;
		CGuiButton bouton_annuler_quitter;
		CGuiButtonImage bouton_camera_par_defaut;
		CGuiButtonImage bouton_quitter;
		CGuiButtonImage bouton_forcer;
		
		
		CGuiLabel label_a_qui_de_jouer;
		CGuiLabel label_confirmation_quitter;
		
		bool doit_on_afficher_fenetre_de_confirmation_pour_quitter;
		
		enum couleurs
		{
			NOIR,
			BLANC
		};
		unsigned char couleur_choisie;// couleur choisie par le joueur 
		unsigned char couleur_en_cours;// couleur qui est en train de jouer
		
		unsigned char mode_de_jeu_choisi;
		
		bool partie_finie;
		unsigned char couleur_vainqueur;
		
		SDL_Thread* IntelligenceArtificielle;
		bool IA_vient_de_finir_de_jouer;
		bool IA_faible_priorite;
		unsigned char IA_difficulte;
		unsigned int IA_temps;
		bool IA_ID;
		bool IA_abandonne;
		void* thread_data[9];// données qui sont envoyées en paramètres au thread
		unsigned char* tableau_des_possibilites_de_deplacement;
		
		bool joueur_blanc_en_echec;
		bool joueur_noir_en_echec;
		
		void initialiser_echiquier();
		void afficher_echiquier();
		void actualiser_camera();
		void clique_sur_l_echiquier(int mouseX, int mouseY);
		void afficher_barre_de_menu(int mouseX, int mouseY);
		void afficher_fleche_de_selection();
		void afficher_fenetre_de_confirmation_pour_quitter();
		bool echec();
		bool echec_et_mat();
		void verifier_tableau_des_possibilites_de_deplacement_en_cas_d_echec(int indice_piece);
		void verifier_echec_apres_coup();
};



#endif

