/**
 * @file scene_menu_principal.cpp
 * @author Frederic MARTIN [martin-frederic@users.sourceforge.net]
 * @brief Implemente E_scene_menu_principal. Voir scenes.h pour la liste de toutes les scenes
 */

 

#include "scene_menu_principal.h"
#include "utils.h"
#include "models.h"

// Instance actuelle de la classe E_scene_menu_principal. Elle est utilisée par
// les fonctions de CALL-BACK car elles ne peuvent pas être membre de la classe
E_scene_menu_principal* instance_actuelle=NULL; 


const int POSITION_X_DES_BOUTONS=500;
const int POSITION_Y_DU_PREMIER_BOUTON=200;

const std::string	MESSAGES_DES_CREDITS[NOMBRE_DE_MESSAGES_DES_CREDITS] = 
{
	"- Main programmer : -",
	"Frederic MARTIN [martin-frederic@users.sourceforge.net]",
	"",
	"",
	"- Main Testers -",
	"Eric LE GUEVEL",	
	"Frederic MARTIN",
	"Helene MARTIN",
	"Jean-Baptiste ZATTARIN",
	"Pierre GOTAB",
	"Vincent MARTIN",
	"Vivien BERNARD",
	"",
	"",
	"- Graphics with : -",
	"Blender",
	"The GIMP",
	"KolourPaint",
	"",
	"",
	"- Special thanks to : -",
	"",
	"",
	"Eric for his Linux tweaks",
	"Jean-Baptiste for testing",
	"Pierre for the camera management",
	"Vivien for AI ideas",
	"Vincent for blender models",
	"Philippe Gilles for all his help",
	"",
	"The KDE team for doing a such great environment",
	"",
	"",
	"And of course all the Linux community !",

};

//--------------------------------------------------//
//--------------------------------------------------//
//    ** fonctions de CALL-BACK  pour le GUI **     //
//--------------------------------------------------//
//--------------------------------------------------//
void sur_bouton_un_joueur()
{
	pGlobalInfos->set_mode_de_jeu(pGlobalInfos->JOUEUR_CONTRE_IA);
	instance_actuelle->prochaine_scene(E_SCENE_JEU_PRINCIPAL);
}

void sur_bouton_deux_joueurs()
{
	pGlobalInfos->set_mode_de_jeu(pGlobalInfos->JOUEUR_CONTRE_JOUEUR);
	instance_actuelle->prochaine_scene(E_SCENE_JEU_PRINCIPAL);
}


void sur_bouton_credits()
{
	instance_actuelle->afficher_credits_init();
}


void sur_bouton_quitter()
{
	pGlobalInfos->quitter_le_jeu();
}




//--------------------------------------------------//
//--------------------------------------------------//
//         **  E_scene_menu_principal **            //
//--------------------------------------------------//
//--------------------------------------------------//
E_scene_menu_principal::E_scene_menu_principal()
{
	fontTexture=NULL;
	instance_actuelle=this;
}

E_scene_menu_principal::~E_scene_menu_principal()
{

}



void E_scene_menu_principal::afficher_echiquier()
{
	
	// matériau "gold" trouvée sur http://devernay.free.fr/cours/opengl/materials.html
	const GLfloat material_plateau_Ka[] = {0.24725f,0.1995f,0.0745f,1.0f};
	const GLfloat material_plateau_Kd[] = {0.75164f,0.60648f,0.22648f,1.0f};
	const GLfloat material_plateau_Ks[] = {0.628281f,0.555802f,0.366065f,1.0f};
	const GLfloat material_plateau_Ke[] = {0.0f,0.0f,0.0f,1.0f};
	// matériau "green rubber" trouvée sur http://devernay.free.fr/cours/opengl/materials.html
	const GLfloat material_table_Ka[] = {0.0f,0.05f,0.0f,1.0f};
	const GLfloat material_table_Kd[] = {0.4f,0.5f,0.4f,1.0f};
	const GLfloat material_table_Ks[] = {0.04f,0.7f,0.04f,1.0f};
	const GLfloat material_table_Ke[] = {0.0f,0.0f,0.0f,1.0f};
	
	// Positionne la caméra
	angleCameraX+=0.0003f*pGlobalInfos->tempsEcoule();
	
	GLfloat cameraDist = 20.0f;
	GLfloat cameraX=cos(angleCameraX)*cos(angleCameraZ)*cameraDist+7.0f;
	GLfloat cameraY=sin(angleCameraZ)*cameraDist;
	GLfloat cameraZ=sin(angleCameraX)*cos(angleCameraZ)*cameraDist+7.0f;
	
	
	// Positionne la caméra
	gluLookAt(cameraX, cameraY, cameraZ,	// coordonnées de la position de la caméra
			  7.0f, 0.0f, 7.0f,			// coordonnées du centre de l'échiquier
			  0.0f, 1.0f, 0.0f);			// caméra tournée selon les y
	
	glEnable(GL_TEXTURE_2D);
	glDisable(GL_LIGHTING);
	
	
	glColor3f(1.0f,1.0f,1.0f);
	damier->utiliser();
	glCallList(liste_des_modeles+OBJ_DAMIER);
	
	if (pGlobalInfos->get_sol_disco())
		glColor3f(0.5f+0.5f*sin(SDL_GetTicks()/1500.0f),0.5f+0.5f*sin((10+SDL_GetTicks())/800.0f),0.5f+0.5f*sin(SDL_GetTicks()/1000.0f));
	
	textureSol->utiliser();
	glCallList(liste_des_modeles+OBJ_SOL);

	glDisable(GL_TEXTURE_2D);
	glEnable(GL_LIGHTING);
	
	
	const GLfloat lightPosition1    [] = { 7.0f, 10.0f, 7.0f, 0.0f };	// position de la source de lumière
	glLightfv(GL_LIGHT1, GL_POSITION, lightPosition1);
	
	// affichage du plateau
	glMaterialfv(GL_FRONT,GL_AMBIENT,material_plateau_Ka);
	glMaterialfv(GL_FRONT,GL_DIFFUSE,material_plateau_Kd);
	glMaterialfv(GL_FRONT,GL_SPECULAR,material_plateau_Ks);
	glMaterialfv(GL_FRONT,GL_EMISSION,material_plateau_Ke);
	glMaterialf(GL_FRONT,GL_SHININESS,51.2f);//51.2f = 128.0f*0.4f
		
	glPushMatrix();
	glTranslatef(7.0f,-0.1f,7.0f);
	glCallList(liste_des_modeles+OBJ_PLATEAU);
	glPopMatrix();
	
	// affichage de la table
	glMaterialfv(GL_FRONT,GL_AMBIENT,material_table_Ka);
	glMaterialfv(GL_FRONT,GL_DIFFUSE,material_table_Kd);
	glMaterialfv(GL_FRONT,GL_SPECULAR,material_table_Ks);
	glMaterialfv(GL_FRONT,GL_EMISSION,material_table_Ke);
	glMaterialf(GL_FRONT,GL_SHININESS,100.0f);//100.0f = 128.0f*0.78125ff
	
	glPushMatrix();
	glTranslatef(5.2f,-1.7f,7.1f);
	glCallList(liste_des_modeles+OBJ_TABLE);
	glPopMatrix();
	
	glPushMatrix();
	glTranslatef(6.0f,0.02f,2.0f);
	glCallList(liste_des_modeles+OBJ_ROI);
	glPopMatrix();	
	
	glPushMatrix();
	glTranslatef(7.5f,0.3f,2.0f);
	glRotatef(16.0f,0.0f,0.0f,1.0f);
	glCallList(liste_des_modeles+OBJ_DAME);
	glPopMatrix();
	
	glPushMatrix();
	glTranslatef(12.5f,0.02f,2.0f);
	glCallList(liste_des_modeles+OBJ_PION);
	glPopMatrix();	
	
	glPushMatrix();
	glTranslatef(9.0f,0.02f,8.0f);
	glCallList(liste_des_modeles+OBJ_PION);
	glPopMatrix();	
	
	glPushMatrix();
	glTranslatef(5.0f,0.02f,12.5f);
	glCallList(liste_des_modeles+OBJ_PION);
	glPopMatrix();	
	
	glPushMatrix();
	glTranslatef(11.0f,0.02f,11.5f);
	glCallList(liste_des_modeles+OBJ_PION);
	glPopMatrix();	
	
	glPushMatrix();
	glTranslatef(2.0f,0.02f,11.5f);
	glCallList(liste_des_modeles+OBJ_TOUR);
	glPopMatrix();	
	
	glDisable(GL_LIGHTING);
	
}


bool E_scene_menu_principal::charger()
{
	if (deja_charge)
		return true;
	
	
	
	if (charger_objets_3d()==false)
	{
		printf("3D object loading failed...\n");
		return false;
	}
	
	textureSol = new Texture;
	if (false==textureSol->charger(E_DONNEES_TEXTURE_SOL,true))
	{
		printf("Loading of :"E_DONNEES_TEXTURE_SOL" failed\n");
		return false;
	}
	textureSol->SetTexParameter(GL_TEXTURE_WRAP_S,GL_REPEAT);
	textureSol->SetTexParameter(GL_TEXTURE_WRAP_T,GL_REPEAT);
	textureSol->SetTexParameter(GL_MODULATE,0);
	
	damier = new Texture;
	if (false==damier->charger(E_DONNEES_TEXTURE_DAMIER,false))
	{
		printf("Loading of :"E_DONNEES_TEXTURE_DAMIER" failed\n");
		return false;
	}
	
	damier->SetTexParameter(GL_TEXTURE_WRAP_S,GL_CLAMP_TO_EDGE); // pour avoir une bordure "lisse" (pas de répétition sur les S)
	damier->SetTexParameter(GL_TEXTURE_WRAP_T,GL_CLAMP_TO_EDGE); // pour avoir une bordure "lisse" (pas de répétition sur les T)
	
	texture_bouton_un_joueur = new Texture;
	if (false==texture_bouton_un_joueur->charger(E_DONNEES_TEXTURE_BOUTON_1_JOUEUR,false))
	{
		printf("Loading of :"E_DONNEES_TEXTURE_BOUTON_1_JOUEUR" failed\n");
		return false;
	}
	
	texture_bouton_deux_joueurs = new Texture;
	if (false==texture_bouton_deux_joueurs->charger(E_DONNEES_TEXTURE_BOUTON_2_JOUEURS,false))
	{
		printf("Loading of :"E_DONNEES_TEXTURE_BOUTON_2_JOUEURS" failed\n");
		return false;
	}
	
	texture_bouton_credits = new Texture;
	if (false==texture_bouton_credits->charger(E_DONNEES_TEXTURE_BOUTON_CREDITS,false))
	{
		printf("Loading of :"E_DONNEES_TEXTURE_BOUTON_CREDITS" failed\n");
		return false;
	}
	
	texture_bouton_quitter = new Texture;
	if (false==texture_bouton_quitter->charger(E_DONNEES_TEXTURE_BOUTON_QUITTER2,false))
	{
		printf("Loading of :"E_DONNEES_TEXTURE_BOUTON_QUITTER2" failed\n");
		return false;
	}
	
	texture_blender = new Texture;
	if (false==texture_blender->charger(E_DONNEES_TEXTURE_ICONE_BLENDER,false))
	{
		printf("Loading of :"E_DONNEES_TEXTURE_ICONE_BLENDER" failed\n");
		return false;
	}
	
	texture_kate = new Texture;
	if (false==texture_kate->charger(E_DONNEES_TEXTURE_ICONE_KATE,false))
	{
		printf("Loading of :"E_DONNEES_TEXTURE_ICONE_KATE" failed\n");
		return false;
	}

	texture_gimp = new Texture;
	if (false==texture_gimp->charger(E_DONNEES_TEXTURE_ICONE_GIMP,false))
	{
		printf("Loading of :"E_DONNEES_TEXTURE_ICONE_GIMP" failed\n");
		return false;
	}

	texture_tux = new Texture;
	if (false==texture_tux->charger(E_DONNEES_TEXTURE_ICONE_TUX,false))
	{
		printf("Loading of :"E_DONNEES_TEXTURE_ICONE_TUX" failed\n");
		return false;
	}
	
	fontTexture = new Texture;
	if (false==fontTexture->charger(E_DONNEES_TEXTURE_FONT,false))
	{
		printf("Loading of :"E_DONNEES_TEXTURE_FONT" failed\n");
		return false;
	}
	
	titre = new Texture;
	if (false==titre->charger(E_DONNEES_TEXTURE_TITRE,false))
	{
		printf("Loading of :"E_DONNEES_TEXTURE_TITRE" failed\n");
		return false;
	}
	
	
	un_gui.Init(1024.0f,768.0f, fontTexture->GetId() );
	
	liste_des_modeles=genererla_liste_des_modeles();	// Crée les display lists des pièces
	
	glEnable(GL_LIGHTING); // activation de la lumière
	glEnable(GL_LIGHT0);// activation de la lumière 0
	glEnable(GL_LIGHT1);// activation de la lumière 1

	// définition des paramètres pour la lumière 0
	const GLfloat lightPosition0    [] = { 7.0f, 10.0f, 7.0f, 0.0f };	// position de la source de lumière
	const GLfloat lightColorAmbient [] = { 0.3f, 0.3f, 0.3f, 1.0f };	// couleur ambiante
	const GLfloat lightColorDiffuse [] = { 1.0f, 1.0f, 1.0f, 1.0f };	// couleur Diffuse
	const GLfloat lightColorSpecular[] = { 1.0f, 1.0f, 1.0f, 1.0f };	// couleur spéculaire	
	
	glLightfv(GL_LIGHT0, GL_POSITION, lightPosition0);
	glLightfv(GL_LIGHT0, GL_AMBIENT,  lightColorAmbient);
	glLightfv(GL_LIGHT0, GL_DIFFUSE,  lightColorDiffuse);
	glLightfv(GL_LIGHT0, GL_SPECULAR, lightColorSpecular);
	
	glLightfv(GL_LIGHT1, GL_AMBIENT,  lightColorAmbient);
	glLightfv(GL_LIGHT1, GL_DIFFUSE,  lightColorDiffuse);
	glLightfv(GL_LIGHT1, GL_SPECULAR, lightColorSpecular);

	
	bouton_un_joueur.SetPos(POSITION_X_DES_BOUTONS,POSITION_Y_DU_PREMIER_BOUTON+0);
	bouton_un_joueur.OnClick(sur_bouton_un_joueur);
	bouton_un_joueur.SetImage(texture_bouton_un_joueur->GetId());
	bouton_un_joueur.SetSize(64,64);
	un_gui.AddWidget(&bouton_un_joueur);
	
	bouton_deux_joueurs.SetPos(POSITION_X_DES_BOUTONS,POSITION_Y_DU_PREMIER_BOUTON+130);
	bouton_deux_joueurs.OnClick(sur_bouton_deux_joueurs);
	bouton_deux_joueurs.SetImage(texture_bouton_deux_joueurs->GetId());
	bouton_deux_joueurs.SetSize(64,64);
	un_gui.AddWidget(&bouton_deux_joueurs);
	
	bouton_credits.SetPos(POSITION_X_DES_BOUTONS,POSITION_Y_DU_PREMIER_BOUTON+260);
	bouton_credits.OnClick(sur_bouton_credits);
	bouton_credits.SetImage(texture_bouton_credits->GetId());
	bouton_credits.SetSize(64,64);
	un_gui.AddWidget(&bouton_credits);
	
	bouton_quitter.SetPos(POSITION_X_DES_BOUTONS,POSITION_Y_DU_PREMIER_BOUTON+390);
	bouton_quitter.OnClick(sur_bouton_quitter);
	bouton_quitter.SetImage(texture_bouton_quitter->GetId());
	bouton_quitter.SetSize(64,64);
	un_gui.AddWidget(&bouton_quitter);
	
	label_un_joueur.SetText("One player");
	label_un_joueur.SetPos(POSITION_X_DES_BOUTONS-65,POSITION_Y_DU_PREMIER_BOUTON+60);
	un_gui.AddWidget(&label_un_joueur);
	
	label_deux_joueurs.SetText("Two players");
	label_deux_joueurs.SetPos(POSITION_X_DES_BOUTONS-80,POSITION_Y_DU_PREMIER_BOUTON+190);
	un_gui.AddWidget(&label_deux_joueurs);

	label_credits.SetText("About");
	label_credits.SetPos(POSITION_X_DES_BOUTONS-10,POSITION_Y_DU_PREMIER_BOUTON+320);
	un_gui.AddWidget(&label_credits);
	
	label_quitter.SetText("Quit");
	label_quitter.SetPos(POSITION_X_DES_BOUTONS-5,POSITION_Y_DU_PREMIER_BOUTON+450);
	un_gui.AddWidget(&label_quitter);
	
	titre_info.SetPos(0,720);
	titre_info.SetSize(1024,30);
	un_gui.AddWidget(&titre_info);
	
	numero_version.SetText("V"VERSION_DU_JEU);
	numero_version.SetPos(750,100);
	un_gui.AddWidget(&numero_version);
	
	for(int i=0;i<NOMBRE_DE_MESSAGES_DES_CREDITS;i++)
	{
		credits_labels[i] = new CGuiLabel(MESSAGES_DES_CREDITS[i]);
		// Si la chaine commence par '-' alors c'est un titre et
		// on le change de couleur
		if (MESSAGES_DES_CREDITS[i][0]=='-')
			credits_labels[i]->SetColor(0.0f,0.0f,1.0f);
		// On met une très grosse largeur de manière à ce que le texte
		// soit centré.
		credits_labels[i]->SetSize(1024,20);
	}
	
	doit_afficher_credits=false;
	doit_afficher_menu=true;
	fade_out=true;
	fade_in=false;
	fade_out_valeur=1.0f;
	fade_in_valeur=0.0f;
	angleCameraX=1.0f;
	angleCameraZ=0.8f;
	scene_suivante=E_SCENE_MENU_PRINCIPAL;
	//---------------------------//
	
	fin_du_chargement();
	return true;
}

void E_scene_menu_principal::decharger()
{
	if (deja_charge==false)
		return;
	
	decharger_objets_3d();
	
	delete damier;
	damier=NULL;
	
	delete textureSol;
	textureSol=NULL;
	
	for(int i=0;i<NOMBRE_DE_MESSAGES_DES_CREDITS;i++)
	{
		delete credits_labels[i];
		credits_labels[i]=NULL;
	}
	
	un_gui.Destroy();
	
	delete fontTexture;
	fontTexture=NULL;
	
	delete texture_bouton_un_joueur;
	texture_bouton_un_joueur=NULL;
	
	delete texture_bouton_deux_joueurs;
	texture_bouton_deux_joueurs=NULL;
	
	delete texture_bouton_credits;
	texture_bouton_credits=NULL;
	
	delete texture_bouton_quitter;
	texture_bouton_quitter=NULL;
	
	delete texture_blender;
	texture_blender=NULL;
	
	delete texture_kate;
	texture_kate=NULL;
	
	delete texture_gimp;
	texture_gimp=NULL;
	
	delete texture_tux;
	texture_tux=NULL;
	
	delete titre;
	titre=NULL;
	
	deja_charge=false;
}

void E_scene_menu_principal::actualiser()
{
	if (deja_charge==false)
	{
		if ( charger()==false )
		{
			printf("Loading of 'E_scene_menu_principal' failed: exiting...\n");
			pGlobalInfos->quitter_le_jeu();
			return;
		}
	}
	
	
	afficher_echiquier();
	
	aller_en_2d();
	
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glEnable(GL_BLEND);
	
	glColor4f(0.0f,0.0f,0.0f,0.55f);
	dessiner_rectangle_plein(0,0,1024,768);
	
	glColor3f(1.0f,1.0f,1.0f);
	dessiner_image(10,700,64,64,texture_blender);
	dessiner_image(80,700,64,64,texture_gimp);
	dessiner_image(880,700,64,64,texture_kate);
	dessiner_image(950,700,64,64,texture_tux);
	dessiner_image(256,20,512,128,titre);
	
	if (doit_afficher_menu)
		afficher_menu();
	else if (doit_afficher_credits)
		afficher_credits();
	
	if (fade_in==false)
	{
		GLfloat largeur=(GLfloat)pGlobalInfos->largeurEcran();
		GLfloat hauteur=(GLfloat)pGlobalInfos->hauteurEcran();
		int x=(int) (1024.0f*pGlobalInfos->SourisX()/largeur);
		int y=(int) (768.0f*pGlobalInfos->SourisY()/hauteur);
		
		if (y>700)
		{
			if (x<80)
				titre_info.SetText("3D models with blender");
			else
			if (x>=80 && x<144)
				titre_info.SetText("Image manipulation with The GIMP");
			else
			if (x>=880 && x<950)
				titre_info.SetText("Written with Kate");
			else
			if (x>=950)
				titre_info.SetText("Developed with Linux");
			else
				titre_info.SetText("(C) 2005-2006 Frederic MARTIN");
		}
		else
			titre_info.SetText("(C) 2005-2006 Frederic MARTIN");
				  	
		un_gui.Update(x,y,pGlobalInfos->cliqueGaucheEnfonce());
	}
	else
		un_gui.Update();
	
	quitter_la_2d();

}


void E_scene_menu_principal::afficher_menu_init()
{
	doit_afficher_credits=false;
	doit_afficher_menu=true;
	
	for(int i=0;i<NOMBRE_DE_MESSAGES_DES_CREDITS;i++)
		un_gui.RemoveWidget(credits_labels[i]);
	
	un_gui.AddWidget(&bouton_un_joueur);
	un_gui.AddWidget(&bouton_deux_joueurs);
	un_gui.AddWidget(&bouton_credits);
	un_gui.AddWidget(&bouton_quitter);
	un_gui.AddWidget(&label_un_joueur);
	un_gui.AddWidget(&label_deux_joueurs);
	un_gui.AddWidget(&label_credits);
	un_gui.AddWidget(&label_quitter);
	
}


void E_scene_menu_principal::afficher_credits_init()
{
	doit_afficher_menu=false;
	doit_afficher_credits=true;
	
	un_gui.RemoveWidget(&bouton_un_joueur);
	un_gui.RemoveWidget(&bouton_deux_joueurs);
	un_gui.RemoveWidget(&bouton_credits);
	un_gui.RemoveWidget(&bouton_quitter);
	un_gui.RemoveWidget(&label_un_joueur);
	un_gui.RemoveWidget(&label_deux_joueurs);
	un_gui.RemoveWidget(&label_credits);
	un_gui.RemoveWidget(&label_quitter);
	
	credits_position=0;
	
	for(int i=0;i<NOMBRE_DE_MESSAGES_DES_CREDITS;i++)
	{
		credits_labels[i]->SetAlpha(0.0f);//on les cache
		un_gui.AddWidget(credits_labels[i]);
	}

}


void E_scene_menu_principal::afficher_menu()
{
	
	// actualise le fade_out
	if (fade_out)
	{
		glColor4f(0.0f,0.0f,0.0f,fade_out_valeur);
		dessiner_rectangle_plein(0,0,1024,768);
		fade_out_valeur-=pGlobalInfos->tempsEcoule()/250.0f;
		if (fade_out_valeur<=0.0f)
			fade_out=false;
	}
	
	// actualise le fade_in
	if (fade_in)
	{
		glColor4f(0.0f,0.0f,0.0f,fade_in_valeur);
		dessiner_rectangle_plein(0,0,1024,768);
		fade_in_valeur+=pGlobalInfos->tempsEcoule()/250.0f;
		if (fade_in_valeur>=1.0f)
		{
			quitter_la_2d();
			effaceEcran();
			SDL_Delay(100);
			decharger();
			pGlobalInfos->changer_de_scene(scene_suivante);
			return;
		}
	}
	
	if (scene_suivante!=E_SCENE_MENU_PRINCIPAL)
		fade_in=true;
	
	// si la touche ECHAP est enfoncée on quitte le jeu
	if (pGlobalInfos->touche_est_enfonce(SDLK_ESCAPE))
	{
		pGlobalInfos->relacher_touches();
		pGlobalInfos->quitter_le_jeu();
	}

	SDL_Delay(20);
}


void E_scene_menu_principal::afficher_credits()
{
	int hauteur_label=30;
	int debut_fade_in=30;// ne pas changer 30 !
	int debut_fade_out=450;
	
	credits_position++;
	
	for(int i=0;i<NOMBRE_DE_MESSAGES_DES_CREDITS;i++)
	{
		credits_labels[i]->SetPos(0,POSITION_Y_DU_PREMIER_BOUTON+debut_fade_out-credits_position+hauteur_label*i);
		int y=credits_labels[i]->getY();
		if (y>POSITION_Y_DU_PREMIER_BOUTON+debut_fade_out)
			credits_labels[i]->SetAlpha(0.0f);
		if (y<POSITION_Y_DU_PREMIER_BOUTON-debut_fade_in)
			credits_labels[i]->SetAlpha(0.0f);		
		if (y>POSITION_Y_DU_PREMIER_BOUTON+debut_fade_out-debut_fade_in && y<POSITION_Y_DU_PREMIER_BOUTON+debut_fade_out)
			credits_labels[i]->SetAlpha((POSITION_Y_DU_PREMIER_BOUTON+debut_fade_out-(float)y)/30.0f);
		if (y>POSITION_Y_DU_PREMIER_BOUTON-debut_fade_in && y<POSITION_Y_DU_PREMIER_BOUTON)
			credits_labels[i]->SetAlpha(1.0f-(((float)POSITION_Y_DU_PREMIER_BOUTON-(float)y)/30.0f));
	}
	

	
	if (credits_position>(debut_fade_in+debut_fade_out+hauteur_label*NOMBRE_DE_MESSAGES_DES_CREDITS))
		afficher_menu_init();
	
	// si la touche ECHAP est enfoncée alors on quitte les crédits
	if (pGlobalInfos->touche_est_enfonce(SDLK_ESCAPE))
	{
		pGlobalInfos->relacher_touches();
		afficher_menu_init();
	}
	
	SDL_Delay(25);// Limite le FPS

}
