/**
 * @file texture.cpp
 * @author Frederic MARTIN [martin-frederic@users.sourceforge.net]
 * @brief Utilise SDL_image pour le chargement d'une image
 */

#include "texture.h"
#include <string.h>


Texture::Texture()
{

}

Texture::~Texture()
{
	decharger();
}



// Charge une image. Formats : BMP, PNM, XPM, LBM, PCX, GIF, JPEG, PNG, TGA. (En fait tous les formats de SDL_image)
bool Texture::charger(const char *fichier, bool mipmap )	
{
	SDL_Surface *surface;
	int pixel;

	surface = IMG_Load(fichier);
	if (surface == NULL)
		return false;
		
	pixel=surface->format->BitsPerPixel;
	largeur=surface->w;
	hauteur=surface->h;
	mipmapping=mipmap;

	glGenTextures(1, &id);
	glBindTexture(GL_TEXTURE_2D, id);


	if (mipmap)
	{
		if ( pixel == 24)
			gluBuild2DMipmaps(GL_TEXTURE_2D, 3, largeur, hauteur, GL_RGB, GL_UNSIGNED_BYTE, surface->pixels);
		else
			gluBuild2DMipmaps(GL_TEXTURE_2D,4,largeur, hauteur, GL_RGBA,GL_UNSIGNED_BYTE,surface->pixels);

		SetTexParameter(GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_LINEAR);
		SetTexParameter(GL_TEXTURE_MAG_FILTER,GL_LINEAR_MIPMAP_LINEAR);
	}
	else
	{
		if ( pixel == 24)
			glTexImage2D(GL_TEXTURE_2D,0,3,largeur, hauteur,0, GL_RGB,GL_UNSIGNED_BYTE,surface->pixels);
		else
			glTexImage2D(GL_TEXTURE_2D,0,4,largeur, hauteur,0, GL_RGBA,GL_UNSIGNED_BYTE,surface->pixels);

		SetTexParameter(GL_TEXTURE_MIN_FILTER,GL_LINEAR);
		SetTexParameter(GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	}

	SDL_FreeSurface(surface);
	surface = NULL;
	
	return true;
}



void Texture::utiliser()
{
	glBindTexture ( GL_TEXTURE_2D, id);
}

void Texture::SetTexParameter (GLenum type, GLint mode)
{
	utiliser();

	glTexParameteri(GL_TEXTURE_2D,
					type,
					mode);
}


void Texture::Set_Blending (GLint mode, GLfloat *env_color)
{
	utiliser();
	if (mode == GL_BLEND)
	{
		glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, (GLfloat) mode);
		glTexEnvfv(GL_TEXTURE_ENV,GL_TEXTURE_ENV_COLOR,(GLfloat*)env_color);
	}
	else
	{
		glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE, (GLfloat) mode);
	}
}

void Texture::Set_Blending (GLint mode)
{
	utiliser();
	glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE, (GLfloat) mode);
}

void Texture::decharger()
{
	glDeleteTextures(1,&id);
}

