/**
 * @file texture.h
 * @author Frederic MARTIN [martin-frederic@users.sourceforge.net]
 * @brief Utilise SDL_image pour le chargement d'une image
 */

#ifndef __TEXTURE_H__
#define __TEXTURE_H__


#include <SDL/SDL.h>
#include <SDL/SDL_opengl.h>
#include <SDL/SDL_image.h>

//! @brief La classe Texture permet de charger des textures en mémoire vidéo.
//! Le nombre de formats supportés est important grace à SDL_image.
class Texture  
{
	public:
		Texture();
		virtual ~Texture();

		//! Charge une image. Formats : BMP, PNM, XPM, LBM, PCX, GIF, JPEG, PNG, TGA. (En fait tous les formats de SDL_image)
		bool charger (const char *fichier, bool mipmap);
		//! Applique la texture : appel de glBindTexture()
		void utiliser();
		void SetTexParameter (GLenum type, GLint mode);
		void Set_Blending  (GLint mode);
		void Set_Blending  (GLint mode , GLfloat *env_color);
		//! Décharge la texture de la mémoire de la carte graphique
		void decharger();
		//! Retroune l'identifiant OpenGL de la texture en mémoire
		inline GLuint GetId(){return id;}

	private:
		GLuint id;
		int largeur;
		int hauteur;
		bool mipmapping;

};


#endif
